@@ -1,9 +1,9 @@
-#!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
+#
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+
 """
 CUSUM stands for cumulative sum, it is a changepoint detection algorithm.
 
@@ -40,7 +40,7 @@ this use case.
 """
 
 import logging
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
 
 import matplotlib.pyplot as plt
 import numpy as np
@@ -50,15 +50,13 @@ from kats.consts import (
     TimeSeriesData,
 )
 from kats.detectors.detector import Detector
-
-# pyre-ignore[21]: Could not find name `chi2` in `scipy.stats`.
 from scipy.stats import chi2  # @manual
 
 
 pd.options.plotting.matplotlib.register_converters = True
 
 # Constants
-CUSUM_DEFAULT_ARGS = {
+CUSUM_DEFAULT_ARGS: Dict[str, Optional[Any]] = {
     "threshold": 0.01,
     "max_iter": 10,
     "delta_std_ratio": 1.0,
@@ -73,17 +71,21 @@ CUSUM_DEFAULT_ARGS = {
     "remove_seasonality": False,
 }
 
-
-def _get_arg(name: str, **kwargs) -> Any:
+# pyre-ignore [3]: Return type must be specified as type that does not contain `Any`.
+def _get_arg(name: str, **kwargs: Any) -> Any:
     return kwargs.get(name, CUSUM_DEFAULT_ARGS[name])
 
 
-class CUSUMMetadata:
-    """CUSUM metadata
+class CUSUMChangePoint(TimeSeriesChangePoint):
+    """CUSUM change point.
 
-    This is the metadata of the changepoint returned by CusumDetectors
+    This is a changepoint detected by CUSUMDetector.
 
     Attributes:
+
+        start_time: Start time of the change.
+        end_time: End time of the change.
+        confidence: The confidence of the change point.
         direction: a str stand for the changepoint change direction 'increase'
             or 'decrease'.
         cp_index: an int for changepoint index.
@@ -101,6 +103,9 @@ class CUSUMMetadata:
 
     def __init__(
         self,
+        start_time: pd.Timestamp,
+        end_time: pd.Timestamp,
+        confidence: float,
         direction: str,
         cp_index: int,
         mu0: Union[float, np.ndarray],
@@ -112,7 +117,8 @@ class CUSUMMetadata:
         stable_changepoint: bool,
         p_value: float,
         p_value_int: float,
-    ):
+    ) -> None:
+        super().__init__(start_time, end_time, confidence)
         self._direction = direction
         self._cp_index = cp_index
         self._mu0 = mu0
@@ -169,12 +175,15 @@ class CUSUMMetadata:
     def p_value_int(self) -> float:
         return self._p_value_int
 
-    def __str__(self) -> str:
+    def __repr__(self) -> str:
         return (
-            f"CUSUMMetadata(direction: {self.direction}, index: {self.cp_index}"
-            f", delta: {self.delta}, regression_detected: "
-            f"{self.regression_detected}, stable_changepoint: "
-            f"{self.stable_changepoint})"
+            f"CUSUMChangePoint(start_time: {self._start_time}, end_time: "
+            f"{self._end_time}, confidence: {self._confidence}, direction: "
+            f"{self._direction}, index: {self._cp_index}, delta: {self._delta}, "
+            f"regression_detected: {self._regression_detected}, "
+            f"stable_changepoint: {self._stable_changepoint}, mu0: {self._mu0}, "
+            f"mu1: {self._mu1}, llr: {self._llr}, llr_int: {self._llr_int}, "
+            f"p_value: {self._p_value}, p_value_int: {self._p_value_int})"
         )
 
 
@@ -184,7 +193,12 @@ class CUSUMDetector(Detector):
     magnitude_ratio: Optional[float] = None
     changes_meta: Optional[Dict[str, Dict[str, Any]]] = None
 
-    def __init__(self, data: TimeSeriesData, is_multivariate: bool = False) -> None:
+    def __init__(
+        self,
+        data: TimeSeriesData,
+        is_multivariate: bool = False,
+        is_vectorized: bool = False,
+    ) -> None:
         """Univariate CUSUM detector for level shifts
 
         Use cusum to detect changes, the algorithm is based on likelihood ratio
@@ -199,11 +213,11 @@ class CUSUMDetector(Detector):
                 MultiCUSUMDetector,
         """
         super(CUSUMDetector, self).__init__(data=data)
-        if not self.data.is_univariate() and not is_multivariate:
+        if not self.data.is_univariate() and not is_multivariate and not is_vectorized:
             msg = (
                 "CUSUMDetector only supports univariate time series, but got "
                 f"{type(self.data.value)}.  For multivariate time series, use "
-                "MultiCUSUMDetector"
+                "MultiCUSUMDetector or VectorizedCUSUMDetector"
             )
             logging.error(msg)
             raise ValueError(msg)
@@ -220,7 +234,8 @@ class CUSUMDetector(Detector):
         if change_direction == "increase":
             changepoint_func = np.argmin
             logging.debug("Detecting increase changepoint.")
-        if change_direction == "decrease":
+        else:
+            assert change_direction == "decrease"
             changepoint_func = np.argmax
             logging.debug("Detecting decrease changepoint.")
         n = 0
@@ -236,6 +251,7 @@ class CUSUMDetector(Detector):
         else:
             changepoint = start_point
 
+        mu0 = mu1 = None
         # iterate until the changepoint converage
         while n < max_iter:
             n += 1
@@ -262,9 +278,9 @@ class CUSUMDetector(Detector):
             delta_int = None
         else:
             llr_int = self._get_llr(
-                ts_int, {"mu0": mu0, "mu1": mu1, "changepoint": changepoint}
+                ts_int,
+                {"mu0": mu0, "mu1": mu1, "changepoint": changepoint},
             )
-            # pyre-ignore[16]: Module `stats` has no attribute `chi2`.
             pval_int = 1 - chi2.cdf(llr_int, 2)
             delta_int = mu1 - mu0
             changepoint += interest_window[0]
@@ -284,7 +300,7 @@ class CUSUMDetector(Detector):
             "delta_int": delta_int,
         }
 
-    def _get_llr(self, ts: np.ndarray, change_meta: Dict[str, Any]):
+    def _get_llr(self, ts: np.ndarray, change_meta: Dict[str, Any]) -> float:
         """
         Calculate the log likelihood ratio
         """
@@ -301,7 +317,7 @@ class CUSUMDetector(Detector):
         mu_tilde, sigma_tilde = np.mean(ts), np.std(ts)
 
         if scale == 0:
-            scale = sigma_tilde
+            scale = sigma_tilde * 0.01
 
         llr = -2 * (
             self._log_llr(ts[: (changepoint + 1)], mu_tilde, sigma_tilde, mu0, scale)
@@ -374,8 +390,7 @@ class CUSUMDetector(Detector):
         return magnitude
 
     # pyre-fixme[14]: `detector` overrides method defined in `Detector` inconsistently.
-    # pyre-fixme[15]: `detector` overrides method defined in `Detector` inconsistently.
-    def detector(self, **kwargs) -> List[Tuple[TimeSeriesChangePoint, CUSUMMetadata]]:
+    def detector(self, **kwargs: Any) -> Sequence[CUSUMChangePoint]:
         """
         Find the change point and calculate related statistics.
 
@@ -407,7 +422,7 @@ class CUSUMDetector(Detector):
                 found, even the insignificant ones.
 
         Returns:
-            A list of tuple of TimeSeriesChangePoint and CUSUMMetadata.
+            A list of CUSUMChangePoint.
         """
         # Extract all arg values or assign defaults from default vals constant
         threshold = _get_arg("threshold", **kwargs)
@@ -435,8 +450,11 @@ class CUSUMDetector(Detector):
             change_directions = ["increase", "decrease"]
 
         for change_direction in change_directions:
-
-            assert change_direction in ["increase", "decrease"]
+            if change_direction not in {"increase", "decrease"}:
+                raise ValueError(
+                    "Change direction must be 'increase' or 'decrease.' "
+                    f"Got {change_direction}"
+                )
 
             change_meta = self._get_change_point(
                 ts,
@@ -445,7 +463,6 @@ class CUSUMDetector(Detector):
                 change_direction=change_direction,
             )
             change_meta["llr"] = self._get_llr(ts, change_meta)
-            # pyre-ignore[16]: Module `stats` has no attribute `chi2`.
             change_meta["p_value"] = 1 - chi2.cdf(change_meta["llr"], 2)
 
             # compare magnitude on interest_window and historical_window
@@ -467,9 +484,7 @@ class CUSUMDetector(Detector):
                         )
                     )
 
-            # pyre-ignore[16]: Module `stats` has no attribute `chi2`.
             if_significant = change_meta["llr"] > chi2.ppf(1 - threshold, 2)
-            # pyre-ignore[16]: Module `stats` has no attribute `chi2`.
             if_significant_int = change_meta["llr_int"] > chi2.ppf(1 - threshold, 2)
             if change_direction == "increase":
                 larger_than_min_abs_change = (
@@ -495,18 +510,16 @@ class CUSUMDetector(Detector):
 
         self.changes_meta = changes_meta
 
-        return self._convert_cusum_changepoints(
-            changes_meta, return_all_changepoints
-        )
+        return self._convert_cusum_changepoints(changes_meta, return_all_changepoints)
 
     def _convert_cusum_changepoints(
         self,
         cusum_changepoints: Dict[str, Dict[str, Any]],
         return_all_changepoints: bool,
-    ) -> List[Tuple[TimeSeriesChangePoint, CUSUMMetadata]]:
+    ) -> List[CUSUMChangePoint]:
         """
         Convert the output from the other kats cusum algorithm into
-        TimeSeriesChangePoint type.
+        CUSUMChangePoint type.
         """
         converted = []
         detected_cps = cusum_changepoints
@@ -515,12 +528,10 @@ class CUSUMDetector(Detector):
             dir_cps = detected_cps[direction]
             if dir_cps["regression_detected"] or return_all_changepoints:
                 # we have a change point
-                change_point = TimeSeriesChangePoint(
+                change_point = CUSUMChangePoint(
                     start_time=dir_cps["changetime"],
                     end_time=dir_cps["changetime"],
                     confidence=1 - dir_cps["p_value"],
-                )
-                metadata = CUSUMMetadata(
                     direction=direction,
                     cp_index=dir_cps["changepoint"],
                     mu0=dir_cps["mu0"],
@@ -533,43 +544,46 @@ class CUSUMDetector(Detector):
                     p_value=dir_cps["p_value"],
                     p_value_int=dir_cps["p_value_int"],
                 )
-                converted.append((change_point, metadata))
+                converted.append(change_point)
 
         return converted
 
     def plot(
-        self, change_points: List[Tuple[TimeSeriesChangePoint, CUSUMMetadata]]
-    ) -> None:
+        self, change_points: Sequence[CUSUMChangePoint], **kwargs: Any
+    ) -> plt.Axes:
         """Plot detection results from CUSUM.
 
         Args:
-            change_points: A list of tuple of TimeSeriesChangePoint and
-            CUSUMMetadata.
+            change_points: A list of CUSUMChangePoint.
+            kwargs: other arguments to pass to subplots.
+
+        Returns:
+            The matplotlib Axes.
         """
         time_col_name = self.data.time.name
         val_col_name = self.data.value.name
 
         data_df = self.data.to_dataframe()
 
-        plt.plot(data_df[time_col_name], data_df[val_col_name])
-
-        if len(change_points) == 0:
-            logging.warning("No change points detected!")
+        _, ax = plt.subplots(**kwargs)
+        ax.plot(data_df[time_col_name], data_df[val_col_name])
 
         for change in change_points:
-            if change[1].regression_detected:
-                plt.axvline(x=change[0].start_time, color="red")
+            if change.regression_detected:
+                ax.axvline(x=change.start_time, color="red")
+        else:
+            logging.warning("No change points detected!")
 
         interest_window = self.interest_window
         if interest_window is not None:
-            plt.axvspan(
+            ax.axvspan(
                 pd.to_datetime(self.data.time)[interest_window[0]],
                 pd.to_datetime(self.data.time)[interest_window[1] - 1],
                 alpha=0.3,
                 label="interets_window",
             )
 
-        plt.show()
+        return ax
 
 
 class MultiCUSUMDetector(CUSUMDetector):
@@ -587,7 +601,7 @@ class MultiCUSUMDetector(CUSUMDetector):
     def __init__(self, data: TimeSeriesData) -> None:
         super(MultiCUSUMDetector, self).__init__(data=data, is_multivariate=True)
 
-    def detector(self, **kwargs) -> List[Tuple[TimeSeriesChangePoint, CUSUMMetadata]]:
+    def detector(self, **kwargs: Any) -> List[CUSUMChangePoint]:
         """
         Overwrite the detector method for MultiCUSUMDetector.
 
@@ -615,7 +629,7 @@ class MultiCUSUMDetector(CUSUMDetector):
 
         # We will always be looking for increases in the CUSUM values for
         # multivariate detection. We keep using change_direction = "increase"
-        # here to have consistent CUSUMMetadata with the univariate detector.
+        # here to be consistent with the univariate detector.
         for change_direction in ["increase"]:
 
             change_meta = self._get_change_point(
@@ -624,10 +638,8 @@ class MultiCUSUMDetector(CUSUMDetector):
                 start_point=start_point,
             )
             change_meta["llr"] = self._get_llr(ts, change_meta)
-            # pyre-ignore[16]: Module `stats` has no attribute `chi2`.
             change_meta["p_value"] = 1 - chi2.cdf(change_meta["llr"], ts.shape[1] + 1)
 
-            # pyre-ignore[16]: Module `stats` has no attribute `chi2`.
             if_significant = change_meta["llr"] > chi2.ppf(
                 1 - threshold, ts.shape[1] + 1
             )
@@ -639,7 +651,7 @@ class MultiCUSUMDetector(CUSUMDetector):
 
         return self._convert_cusum_changepoints(changes_meta, return_all_changepoints)
 
-    def _get_llr(self, ts: np.ndarray, change_meta: Dict[str, Any]):
+    def _get_llr(self, ts: np.ndarray, change_meta: Dict[str, Any]) -> float:
         mu0: float = change_meta["mu0"]
         mu1: float = change_meta["mu1"]
         sigma0: float = change_meta["sigma0"]
@@ -673,7 +685,7 @@ class MultiCUSUMDetector(CUSUMDetector):
         sigma0: Union[float, np.ndarray],
         mu1: Union[float, np.ndarray],
         sigma1: Union[float, np.ndarray],
-    ):
+    ) -> float:
         try:
             sigma0_inverse = np.linalg.inv(sigma0)
             sigma1_inverse = np.linalg.inv(sigma1)
@@ -690,10 +702,12 @@ class MultiCUSUMDetector(CUSUMDetector):
             for i in range(len(x))
         )
 
-    # pyre-fixme[14]: `_get_change_point` overrides method defined in
-    #  `CUSUMDetector` inconsistently.
     def _get_change_point(
-        self, ts: np.ndarray, max_iter: int, start_point: int
+        self,
+        ts: np.ndarray,
+        max_iter: int,
+        start_point: int,
+        change_direction: str = "increase",
     ) -> Dict[str, Any]:
 
         # locate the change point using cusum method
@@ -703,7 +717,7 @@ class MultiCUSUMDetector(CUSUMDetector):
 
         if start_point is None:
             start_point = len(ts_int) // 2
-            changepoint = start_point
+        changepoint = start_point
 
         # iterate until the changepoint converage
         while n < max_iter:
@@ -775,3 +789,173 @@ class MultiCUSUMDetector(CUSUMDetector):
             "p_value_int": pval_int,
             "delta_int": delta_int,
         }
+
+
+class VectorizedCUSUMDetector(CUSUMDetector):
+    """
+    VectorizedCUSUM is the vecteorized version of CUSUM. It can take
+    multiple time series as an input and run CUSUM algorithm on each time series
+    in a vectorized manner.
+
+    Attributes:
+        data: The input time series data from TimeSeriesData
+    """
+
+    changes_meta_list: Optional[List[Dict[str, Dict[str, Any]]]] = None
+
+    def __init__(self, data: TimeSeriesData) -> None:
+        super(VectorizedCUSUMDetector, self).__init__(
+            data=data, is_multivariate=False, is_vectorized=True
+        )
+
+    def detector(self, **kwargs: Any) -> Sequence[CUSUMChangePoint]:
+        msg = "VectorizedCUSUMDetector is in beta and please use detector_()"
+        logging.error(msg)
+        raise ValueError(msg)
+
+    def detector_(self, **kwargs: Any) -> List[List[CUSUMChangePoint]]:
+        """
+        Detector method for vectorized version of CUSUM
+
+        Args:
+            threshold: Optional; float; significance level, default: 0.01.
+            max_iter: Optional; int, maximum iteration in finding the
+                changepoint.
+            change_directions: Optional; list<str>; a list contain either or
+                both 'increase' and 'decrease' to specify what type of change
+                want to detect.
+            return_all_changepoints: Optional; bool; return all the changepoints
+                found, even the insignificant ones.
+
+        Returns:
+            A list of tuple of TimeSeriesChangePoint and CUSUMMetadata.
+        """
+        # Extract all arg values or assign defaults from default vals constant
+        threshold = _get_arg("threshold", **kwargs)
+        max_iter = _get_arg("max_iter", **kwargs)
+        change_directions = _get_arg("change_directions", **kwargs)
+        return_all_changepoints = _get_arg("return_all_changepoints", **kwargs)
+
+        # Use array to store the data
+        ts_all = self.data.value.to_numpy()
+        ts_all = ts_all.astype("float64")
+        changes_meta_list = []
+
+        if change_directions is None:
+            change_directions = ["increase", "decrease"]
+
+        change_meta_all = {}
+        for change_direction in change_directions:
+            if change_direction not in {"increase", "decrease"}:
+                raise ValueError(
+                    "Change direction must be 'increase' or 'decrease.' "
+                    f"Got {change_direction}"
+                )
+
+            change_meta_all[change_direction] = self._get_change_point_multiple_ts(
+                ts_all,
+                max_iter=max_iter,
+                change_direction=change_direction,
+            )
+
+        ret = []
+        for col_idx in np.arange(ts_all.shape[1]):
+            ts = ts_all[:, col_idx]
+            changes_meta = {}
+            for change_direction in change_directions:
+                change_meta_ = change_meta_all[change_direction]
+                change_meta = {
+                    k: change_meta_[k][col_idx]
+                    if isinstance(change_meta_[k], np.ndarray)
+                    or isinstance(change_meta_[k], list)
+                    else change_meta_[k]
+                    for k in change_meta_
+                }
+                change_meta["llr"] = self._get_llr(ts, change_meta)
+                change_meta["p_value"] = 1 - chi2.cdf(change_meta["llr"], 2)
+                change_meta["regression_detected"] = change_meta["p_value"] < threshold
+                changes_meta[change_direction] = change_meta
+            changes_meta_list.append(changes_meta)
+            ret.append(
+                self._convert_cusum_changepoints(changes_meta, return_all_changepoints)
+            )
+        self.changes_meta_list = changes_meta_list
+        return ret
+
+    def _get_change_point_multiple_ts(
+        self, ts: np.ndarray, max_iter: int, change_direction: str
+    ) -> Dict[str, Any]:
+        """
+        Find change points in a list of time series
+        """
+        # locate the change point using cusum method
+        if change_direction == "increase":
+            changepoint_func = np.argmin
+            logging.debug("Detecting increase changepoint.")
+        else:
+            assert change_direction == "decrease"
+            changepoint_func = np.argmax
+            logging.debug("Detecting decrease changepoint.")
+
+        n_ts = ts.shape[1]
+        n_pts = ts.shape[0]
+        n = 0
+        # use the middle point as initial change point to estimate mu0 and mu1
+        ts_int = ts
+        tmp = ts_int - np.tile(np.mean(ts_int, axis=0), (n_pts, 1))
+        cusum_ts = np.cumsum(tmp, axis=0)
+        changepoint = np.minimum(changepoint_func(cusum_ts, axis=0), n_pts - 2)
+        mu0 = mu1 = None
+        stable_changepoint = [False] * len(changepoint)
+        # iterate until the changepoint converage
+        while n < max_iter:
+            mask = np.zeros((n_pts, n_ts), dtype=bool)
+            for i, c in enumerate(changepoint):
+                mask[: (c + 1), i] = True
+            n += 1
+            mu0 = np.divide(
+                np.sum(np.multiply(ts_int, mask), axis=0), np.sum(mask, axis=0)
+            )
+            mu1 = np.divide(
+                np.sum(np.multiply(ts_int, ~mask), axis=0), np.sum(~mask, axis=0)
+            )
+            mean = (mu0 + mu1) / 2
+            # here is where cusum is happening
+            tmp = ts_int - np.tile(mean, (n_pts, 1))
+            cusum_ts = np.cumsum(tmp, axis=0)
+            next_changepoint = np.maximum(
+                np.minimum(changepoint_func(cusum_ts, axis=0), n_pts - 2), 1
+            )
+            stable_changepoint = np.equal(changepoint, next_changepoint)
+            if all(stable_changepoint):
+                break
+            changepoint = next_changepoint
+
+        # llr in interest window, not supported yet
+        llr_int = np.inf
+        pval_int = np.NaN
+        delta_int = None
+
+        # full time changepoint and mean
+        mask = np.zeros((n_pts, n_ts), dtype=bool)
+        changetime = []
+        for i, c in enumerate(changepoint):
+            mask[: (c + 1), i] = True
+            changetime.append(self.data.time[c])
+
+        mu0 = np.divide(np.sum(np.multiply(ts_int, mask), axis=0), np.sum(mask, axis=0))
+        mu1 = np.divide(
+            np.sum(np.multiply(ts_int, ~mask), axis=0), np.sum(~mask, axis=0)
+        )
+
+        return {
+            "changepoint": changepoint,
+            "mu0": mu0,
+            "mu1": mu1,
+            "changetime": changetime,
+            "stable_changepoint": stable_changepoint,
+            "delta": mu1 - mu0,
+            "llr_int": llr_int,
+            "p_value_int": pval_int,
+            "delta_int": delta_int,
+        }
