@@ -1,6 +1,5 @@
-#!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
+#
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
@@ -31,7 +30,7 @@ import json
 import logging
 from datetime import datetime
 from enum import Enum
-from typing import Any, List, Optional, Union
+from typing import Dict, cast, Any, List, Optional, Union
 
 import numpy as np
 import pandas as pd
@@ -39,16 +38,20 @@ from kats.consts import (
     DEFAULT_VALUE_NAME,
     TimeSeriesData,
 )
-from kats.detectors.cusum_detection import CUSUMDetector, CUSUM_DEFAULT_ARGS
+from kats.detectors.cusum_detection import (
+    CUSUMDetector,
+    CUSUM_DEFAULT_ARGS,
+)
 from kats.detectors.detector import DetectorModel
 from kats.detectors.detector_consts import AnomalyResponse
 from kats.utils.decomposition import TimeSeriesDecomposition
 
 
 NORMAL_TOLERENCE = 1  # number of window
-CHANGEPOINT_RETENTION = 7 * 24 * 60 * 60  # in seconds
+CHANGEPOINT_RETENTION: int = 7 * 24 * 60 * 60  # in seconds
 MAX_CHANGEPOINT = 10
 
+
 def percentage_change(
     data: TimeSeriesData, pre_mean: float, **kwargs: Any
 ) -> TimeSeriesData:
@@ -93,14 +96,16 @@ class CusumScoreFunction(Enum):
     percentage_change = "percentage_change"
     z_score = "z_score"
 
+
 # Score Function Constants
+# pyre-fixme
 SCORE_FUNC_DICT = {
     CusumScoreFunction.change.value: change,
     CusumScoreFunction.percentage_change.value: percentage_change,
     CusumScoreFunction.z_score.value: z_score,
 }
-DEFAULT_SCORE_FUNCTION = CusumScoreFunction.change
-STR_TO_SCORE_FUNC = { # Used for param tuning
+DEFAULT_SCORE_FUNCTION: CusumScoreFunction = CusumScoreFunction.change
+STR_TO_SCORE_FUNC: Dict[str, CusumScoreFunction] = {  # Used for param tuning
     "change": CusumScoreFunction.change,
     "percentage_change": CusumScoreFunction.percentage_change,
     "z_score": CusumScoreFunction.z_score,
@@ -133,6 +138,7 @@ class CUSUMDetectorModel(DetectorModel):
         score_func: The score function to calculate the anomaly score.
         remove_seasonality: If apply STL to remove seasonality.
     """
+
     def __init__(
         self,
         serialized_model: Optional[bytes] = None,
@@ -146,38 +152,40 @@ class CUSUMDetectorModel(DetectorModel):
         change_directions: List[str] = CUSUM_DEFAULT_ARGS["change_directions"],
         score_func: Union[str, CusumScoreFunction] = DEFAULT_SCORE_FUNCTION,
         remove_seasonality: bool = CUSUM_DEFAULT_ARGS["remove_seasonality"],
-    ):
+    ) -> None:
         if serialized_model:
             previous_model = json.loads(serialized_model)
-            self.cps = previous_model["cps"]
-            self.alert_fired = previous_model["alert_fired"]
-            self.pre_mean = previous_model["pre_mean"]
-            self.pre_std = previous_model["pre_std"]
-            self.number_of_normal_scan = previous_model["number_of_normal_scan"]
-            self.alert_change_direction = previous_model["alert_change_direction"]
-            self.scan_window = previous_model["scan_window"]
-            self.historical_window = previous_model["historical_window"]
-            self.step_window = previous_model["step_window"]
-            self.threshold = previous_model["threshold"]
-            self.delta_std_ratio = previous_model["delta_std_ratio"]
-            self.magnitude_quantile = previous_model["magnitude_quantile"]
-            self.magnitude_ratio = previous_model["magnitude_ratio"]
-            self.change_directions = previous_model["change_directions"]
-            self.score_func = previous_model["score_func"]
+            self.cps: List[int] = previous_model["cps"]
+            self.alert_fired: bool = previous_model["alert_fired"]
+            self.pre_mean: float = previous_model["pre_mean"]
+            self.pre_std: float = previous_model["pre_std"]
+            self.number_of_normal_scan: int = previous_model["number_of_normal_scan"]
+            self.alert_change_direction: str = previous_model["alert_change_direction"]
+            self.scan_window: int = previous_model["scan_window"]
+            scan_window = previous_model["scan_window"]
+            self.historical_window: int = previous_model["historical_window"]
+            self.step_window: int = previous_model["step_window"]
+            step_window = previous_model["step_window"]
+            self.threshold: float = previous_model["threshold"]
+            self.delta_std_ratio: float = previous_model["delta_std_ratio"]
+            self.magnitude_quantile: float = previous_model["magnitude_quantile"]
+            self.magnitude_ratio: float = previous_model["magnitude_ratio"]
+            self.change_directions: List[str] = previous_model["change_directions"]
+            self.score_func: CusumScoreFunction = previous_model["score_func"]
             if "remove_seasonality" in previous_model:
-                self.remove_seasonality = previous_model["remove_seasonality"]
+                self.remove_seasonality: bool = previous_model["remove_seasonality"]
             else:
-                self.remove_seasonality = remove_seasonality
+                self.remove_seasonality: bool = remove_seasonality
         elif scan_window is not None and historical_window is not None:
             self.cps = []
             self.alert_fired = False
             self.pre_mean = 0
             self.pre_std = 1
             self.number_of_normal_scan = 0
-            self.alert_change_direction = None
+            self.alert_change_direction: Union[str, None] = None
             self.scan_window = scan_window
             self.historical_window = historical_window
-            self.step_window = step_window
+            self.step_window = cast(int, step_window)
             self.threshold = threshold
             self.delta_std_ratio = delta_std_ratio
             self.magnitude_quantile = magnitude_quantile
@@ -195,20 +203,16 @@ class CUSUMDetectorModel(DetectorModel):
 
         else:
             raise ValueError(
-                """
-            You must either provide serialized model or values for scan_window and historical_window.
-            """
+                "You must provide either serialized model or values for "
+                "scan_window and historical_window."
             )
-        # pyre-fixme[58]: `>=` is not supported for operand types `int` and
-        #  `Optional[int]`.
-        # pyre-fixme[58]: `>=` is not supported for operand types `int` and
-        #  `Optional[int]`.
         if step_window is not None and step_window >= scan_window:
             raise ValueError(
-                "Step window should smaller than scan window to ensure we have overlap for scan windows."
+                "Step window should be smaller than scan window to ensure we "
+                "have overlap for scan windows."
             )
 
-    def __eq__(self, other):
+    def __eq__(self, other: object) -> bool:
         if isinstance(other, CUSUMDetectorModel):
             return (
                 self.cps == other.cps
@@ -240,13 +244,15 @@ class CUSUMDetectorModel(DetectorModel):
         self.alert_fired = False
         self.number_of_normal_scan = 0
 
-    def _set_alert_on(self, baseline_mean: float, baseline_std: float, alert_change_direction: str) -> None:
+    def _set_alert_on(
+        self, baseline_mean: float, baseline_std: float, alert_change_direction: str
+    ) -> None:
         self.alert_fired = True
         self.alert_change_direction = alert_change_direction
         self.pre_mean = baseline_mean
         self.pre_std = baseline_std
 
-    def _if_normal(self, cur_mean: float, change_directions: str) -> None:
+    def _if_normal(self, cur_mean: float, change_directions: List[str]) -> bool:
         if change_directions is not None:
             increase, decrease = (
                 "increase" in change_directions,
@@ -272,7 +278,7 @@ class CUSUMDetectorModel(DetectorModel):
         self,
         data: TimeSeriesData,
         historical_data: TimeSeriesData,
-        scan_window: int,
+        scan_window: Union[int, pd.Timedelta],
         threshold: float = CUSUM_DEFAULT_ARGS["threshold"],
         delta_std_ratio: float = CUSUM_DEFAULT_ARGS["delta_std_ratio"],
         magnitude_quantile: float = CUSUM_DEFAULT_ARGS["magnitude_quantile"],
@@ -321,21 +327,20 @@ class CUSUMDetectorModel(DetectorModel):
                 change_directions=change_directions,
             )
             if len(changepoints) > 0:
-                cp, meta = sorted(changepoints, key=lambda x: x[0].start_time)[0]
+                cp = sorted(changepoints, key=lambda x: x.start_time)[0]
                 self.cps.append(int(cp.start_time.value / 1e9))
 
                 if len(self.cps) > MAX_CHANGEPOINT:
                     self.cps.pop(0)
 
                 self._set_alert_on(
-                    historical_data.value[: meta.cp_index + 1].mean(),
-                    historical_data.value[: meta.cp_index + 1].std(),
-                    meta.direction,
+                    historical_data.value[: cp.cp_index + 1].mean(),
+                    historical_data.value[: cp.cp_index + 1].std(),
+                    cp.direction,
                 )
         else:
             cur_mean = historical_data[scan_start_index:].value.mean()
 
-            # pyre-fixme[6]: Expected `str` for 2nd param but got `List[str]`.
             if self._if_normal(cur_mean, change_directions):
                 self.number_of_normal_scan += 1
                 if self.number_of_normal_scan >= NORMAL_TOLERENCE:
@@ -389,12 +394,11 @@ class CUSUMDetectorModel(DetectorModel):
             ),
         )
 
-    # pyre-fixme[14]: `fit_predict` overrides method defined in `DetectorModel`
-    #  inconsistently.
     def fit_predict(
         self,
         data: TimeSeriesData,
         historical_data: Optional[TimeSeriesData] = None,
+        **kwargs: Any,
     ) -> AnomalyResponse:
         """
         This function combines fit and predict and return anomaly socre for data. It
@@ -414,7 +418,7 @@ class CUSUMDetectorModel(DetectorModel):
             historical_data: :class:`kats.consts.TimeSeriesData` object representing the history.
 
         Returns:
-            The anomaly response contains the anomaly socres.
+            The anomaly response contains the anomaly scores.
         """
         # get parameters
         scan_window = self.scan_window
@@ -501,7 +505,6 @@ class CUSUMDetectorModel(DetectorModel):
             # if len(all data) is smaller than historical window return zero score
             return AnomalyResponse(
                 scores=self._predict(smooth_historical_data[-len(data) :], score_func),
-                # pyre-fixme[6]: Expected `ConfidenceBand` for 2nd param but got `None`.
                 confidence_band=None,
                 predicted_ts=None,
                 anomaly_magnitude_ts=self._zeros_ts(data),
@@ -518,7 +521,6 @@ class CUSUMDetectorModel(DetectorModel):
             # if len(all data) is smaller than scan data return zero score
             return AnomalyResponse(
                 scores=self._predict(smooth_historical_data[-len(data) :], score_func),
-                # pyre-fixme[6]: Expected `ConfidenceBand` for 2nd param but got `None`.
                 confidence_band=None,
                 predicted_ts=None,
                 anomaly_magnitude_ts=self._zeros_ts(data),
@@ -563,8 +565,7 @@ class CUSUMDetectorModel(DetectorModel):
             self._fit(
                 in_data,
                 in_hist,
-                # pyre-fixme[6]: Expected `int` for 3rd param but got `Timedelta`.
-                scan_window=scan_window,
+                scan_window=cast(Union[int, pd.Timedelta], scan_window),
                 threshold=threshold,
                 delta_std_ratio=delta_std_ratio,
                 magnitude_quantile=magnitude_quantile,
@@ -594,9 +595,7 @@ class CUSUMDetectorModel(DetectorModel):
             self._fit(
                 in_data,
                 in_hist,
-                # pyre-fixme[6]: Expected `ConfidenceBand` for 2nd param but got `None`.
-                # pyre-fixme[6]: Expected `int` for 3rd param but got `Timedelta`.
-                scan_window=scan_window,
+                scan_window=cast(Union[int, pd.Timedelta], scan_window),
                 threshold=threshold,
                 delta_std_ratio=delta_std_ratio,
                 magnitude_quantile=magnitude_quantile,
@@ -613,7 +612,6 @@ class CUSUMDetectorModel(DetectorModel):
 
         return AnomalyResponse(
             scores=score_tsd,
-            # pyre-fixme[6]: Expected `ConfidenceBand` for 2nd param but got `None`.
             confidence_band=None,
             predicted_ts=None,
             anomaly_magnitude_ts=self._zeros_ts(data),
@@ -633,18 +631,19 @@ class CUSUMDetectorModel(DetectorModel):
         else:
             raise ValueError("direction can only be right or left")
 
-    # pyre-fixme[14]: `fit` overrides method defined in `DetectorModel` inconsistently.
     def fit(
         self,
         data: TimeSeriesData,
         historical_data: Optional[TimeSeriesData] = None,
+        **kwargs: Any,
     ) -> None:
         self.fit_predict(data, historical_data)
 
-    # pyre-fixme[14]: `predict` overrides method defined in `DetectorModel`
-    #  inconsistently.
     def predict(
-        self, data: TimeSeriesData, historical_data: Optional[TimeSeriesData] = None
+        self,
+        data: TimeSeriesData,
+        historical_data: Optional[TimeSeriesData] = None,
+        **kwargs: Any,
     ) -> AnomalyResponse:
         """
         predict is not implemented
