@@ -1,6 +1,5 @@
-#!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
+#
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
@@ -8,7 +7,7 @@ from __future__ import annotations
 
 from dataclasses import dataclass
 from datetime import datetime
-from typing import List, Optional, Tuple, Union
+from typing import List, Optional, Tuple, Union, cast
 
 import attr
 import numpy as np
@@ -148,12 +147,14 @@ class ChangePointInterval:
     def variance_val(self) -> Union[float, ArrayLike]:
         if self.num_series == 1:
             vals = self.data
-            return 0.0 if vals is None else np.var(vals)
+            # the t-test uses the sample standard deviation^2 instead of variance,
+            return 0.0 if vals is None or len(vals) == 1 else np.var(vals, ddof=1)
         else:
             data_df = self.data_df
-            if data_df is None:
+            if data_df is None or len(data_df) == 1:
                 return np.zeros(self.num_series)
-            return np.array([np.var(data_df[c].values) for c in self._ts_cols])
+            # the t-test uses the sample standard deviation^2 instead of variance,
+            return np.array([np.var(data_df[c].values, ddof=1) for c in self._ts_cols])
 
     def __len__(self) -> int:
         df = self.data_df
@@ -170,12 +171,36 @@ class ChangePointInterval:
 
 # Percentage Change Object
 class PercentageChange:
+    """
+    PercentageChange is a class which is widely used in detector models. It calculates how much current TS changes
+    compared to previous historical TS. It includes method to calculate t-scores, upper bound, lower bound, etc., for
+    Statsig Detector model to detect significant change.
+
+    Attributes:
+        current: ChangePointInterval. The TS interval we'd like to detect.
+        previous: ChangePointInterval. The historical TS interval, which we compare the current TS interval with.
+        method: str, default value is "fdr_bh".
+        skip_rescaling: bool, default value is False. For multi-variate TS, we need to rescale p-values so that alpha is still the threshold for rejection.
+                    For Statsig detector, when a given TS (except historical part) is longer than max_split_ts_length,
+                    we will transform this long univariate TS into a multi-variate TS and then use multistatsig detector instead.
+                    In this case, we should skip rescaling p-values.
+        use_corrected_scores: bool, default value is False, using original t-scores or correct t-scores.
+    """
+
+    upper: Optional[Union[float, np.ndarray]]
+    lower: Optional[Union[float, np.ndarray]]
+    _t_score: Optional[Union[float, np.ndarray]]
+    _p_value: Optional[Union[float, np.ndarray]]
+    num_series: int
+
     def __init__(
         self,
         current: ChangePointInterval,
         previous: ChangePointInterval,
-        method="fdr_bh",
-    ):
+        method: str = "fdr_bh",
+        skip_rescaling: bool = False,
+        use_corrected_scores: bool = False,
+    ) -> None:
         self.current = current
         self.previous = previous
 
@@ -187,6 +212,12 @@ class PercentageChange:
         self.method = method
         self.num_series = self.current.num_series
 
+        # If we'd like skip rescaling p-values for multivariate timeseires data
+        self.skip_rescaling = skip_rescaling
+
+        # 2 t scores strategies
+        self.use_corrected_scores = use_corrected_scores
+
     @property
     def ratio_estimate(self) -> Union[float, np.ndarray]:
         # pyre-ignore[6]: Expected float for 1st positional only parameter to call float.__truediv__ but got Union[float, np.ndarray].
@@ -200,13 +231,13 @@ class PercentageChange:
     def perc_change_upper(self) -> float:
         if self.upper is None:
             self._delta_method()
-        return (self.upper - 1) * 100.0
+        return (cast(Union[float, np.ndarray], self.upper) - 1) * 100.0
 
     @property
     def perc_change_lower(self) -> float:
         if self.lower is None:
             self._delta_method()
-        return (self.lower - 1) * 100.0
+        return (cast(Union[float, np.ndarray], self.lower) - 1) * 100.0
 
     @property
     def direction(self) -> Union[str, ArrayLike]:
@@ -224,26 +255,30 @@ class PercentageChange:
         if self.num_series > 1:
             return np.array(
                 [
-                    False if self.upper[i] > 1.0 and self.lower[i] < 1 else True
+                    False
+                    if cast(np.ndarray, self.upper)[i] > 1.0
+                    and cast(np.ndarray, self.lower)[i] < 1
+                    else True
                     for i in range(self.current.num_series)
                 ]
             )
         # not stat sig e.g. [0.88, 1.55]
-        return not (self.upper > 1.0 and self.lower < 1.0)
+        return not (
+            cast(Union[float, np.ndarray], self.upper) > 1.0
+            and cast(Union[float, np.ndarray], self.lower) < 1.0
+        )
 
     @property
     def score(self) -> float:
         if self._t_score is None:
             self._ttest()
-
-        return self._t_score
+        return cast(float, self._t_score)
 
     @property
     def p_value(self) -> float:
         if self._p_value is None:
             self._ttest()
-
-        return self._p_value
+        return cast(float, self._p_value)
 
     @property
     def mean_previous(self) -> Union[float, np.ndarray]:
@@ -304,9 +339,12 @@ class PercentageChange:
         # pyre-ignore[58]: * is not supported for operand types int and Union[float, np.ndarray].
         s_p = np.sqrt(((n_1 - 1) * s_1_sq + (n_2 - 1) * s_2_sq) / (n_1 + n_2 - 2))
 
-        # s_p_mean = s_p * np.sqrt((1. / n_1) + (1./ n_2))
+        if not self.use_corrected_scores:
+            return s_p
 
-        return s_p
+        # based on the definition of t-test, we should return s_p_mean
+        s_p_mean = s_p * np.sqrt((1.0 / n_1) + (1.0 / n_2))
+        return s_p_mean
 
     def _ttest_manual(self) -> Tuple[float, float]:
         """
@@ -378,17 +416,27 @@ class PercentageChange:
                     current_slice, prev_slice, equal_var=True, nan_policy="omit"
                 )
 
+        # if un-scaled t_score and p_value are needed
+        if self.skip_rescaling:
+            self._p_value = p_value_start
+            self._t_score = t_value_start
+            return
+
         # The new p-values are the old p-values rescaled so that self.alpha is still the threshold for rejection
         _, self._p_value, _, _ = multitest.multipletests(
             p_value_start, alpha=self.alpha, method=self.method
         )
         self._t_score = np.zeros(num_series)
         # We are using a two-sided test here, so we take inverse_tcdf(self._p_value / 2) with df = len(self.current) + len(self.previous) - 2
+
+        _t_score: np.ndarray = self._t_score
+        _p_value: np.ndarray = cast(np.ndarray, self._p_value)
         for i in range(self.current.num_series):
             if t_value_start[i] < 0:
-                self._t_score[i] = t.ppf(self._p_value[i] / 2, self._get_df())
+                _t_score[i] = t.ppf(_p_value[i] / 2, self._get_df())
             else:
-                self._t_score[i] = t.ppf(1 - self._p_value[i] / 2, self._get_df())
+                _t_score[i] = t.ppf(1 - _p_value[i] / 2, self._get_df())
+        self._t_score = _t_score
 
     def _calc_cov(self) -> float:
         """
@@ -401,8 +449,13 @@ class PercentageChange:
         n_min = min(len(current), len(previous))
         if n_min == 0:
             return np.nan
-        current = current[-n_min:-1]
-        previous = previous[-n_min:-1]
+
+        current = current[-n_min:]
+        previous = previous[-n_min:]
+
+        # for multivariate TS data
+        if self.num_series > 1:
+            return np.asarray([np.cov(current[:, c], previous[:, c])[0, 1] / n_min for c in range(self.num_series)])
 
         return np.cov(current, previous)[0, 1] / n_min
 
@@ -438,14 +491,18 @@ class ConfidenceBand:
 
 
 class AnomalyResponse:
+
+    key_mapping: List[str]
+    num_series: int
+
     def __init__(
         self,
         scores: TimeSeriesData,
-        confidence_band: ConfidenceBand,
-        predicted_ts: TimeSeriesData,
+        confidence_band: Optional[ConfidenceBand],
+        predicted_ts: Optional[TimeSeriesData],
         anomaly_magnitude_ts: TimeSeriesData,
-        stat_sig_ts: TimeSeriesData,
-    ):
+        stat_sig_ts: Optional[TimeSeriesData],
+    ) -> None:
         self.scores = scores
         self.confidence_band = confidence_band
         self.predicted_ts = predicted_ts
@@ -473,16 +530,22 @@ class AnomalyResponse:
         Add one more point and remove the last point
         """
         self.scores = self._update_ts_slice(self.scores, time, score)
-        self.confidence_band = ConfidenceBand(
-            lower=self._update_ts_slice(self.confidence_band.lower, time, ci_lower),
-            upper=self._update_ts_slice(self.confidence_band.upper, time, ci_upper),
-        )
+        confidence_band = self.confidence_band
+        if confidence_band is not None:
+            self.confidence_band = ConfidenceBand(
+                lower=self._update_ts_slice(confidence_band.lower, time, ci_lower),
+                upper=self._update_ts_slice(confidence_band.upper, time, ci_upper),
+            )
 
-        self.predicted_ts = self._update_ts_slice(self.predicted_ts, time, pred)
+        predicted_ts = self.predicted_ts
+        if predicted_ts is not None:
+            self.predicted_ts = self._update_ts_slice(predicted_ts, time, pred)
         self.anomaly_magnitude_ts = self._update_ts_slice(
             self.anomaly_magnitude_ts, time, anom_mag
         )
-        self.stat_sig_ts = self._update_ts_slice(self.stat_sig_ts, time, stat_sig)
+        stat_sig_ts = self.stat_sig_ts
+        if stat_sig_ts is not None:
+            self.stat_sig_ts = self._update_ts_slice(stat_sig_ts, time, stat_sig)
 
     def _update_ts_slice(
         self, ts: TimeSeriesData, time: datetime, value: Union[float, ArrayLike]
@@ -530,45 +593,64 @@ class AnomalyResponse:
         Add one more point and remove the last point
         """
         self._inplace_update_ts(self.scores, time, score)
-        self._inplace_update_ts(self.confidence_band.lower, time, ci_lower),
-        self._inplace_update_ts(self.confidence_band.upper, time, ci_upper)
+        cb = self.confidence_band
+        if cb is not None:
+            self._inplace_update_ts(cb.lower, time, ci_lower),
+            self._inplace_update_ts(cb.upper, time, ci_upper)
 
         self._inplace_update_ts(self.predicted_ts, time, pred)
         self._inplace_update_ts(self.anomaly_magnitude_ts, time, anom_mag)
         self._inplace_update_ts(self.stat_sig_ts, time, stat_sig)
 
     def _inplace_update_ts(
-        self, ts: TimeSeriesData, time: datetime, value: Union[float, ArrayLike]
+        self,
+        ts: Optional[TimeSeriesData],
+        time: datetime,
+        value: Union[float, ArrayLike],
     ) -> None:
+        if ts is None:
+            return
         if self.num_series == 1:
             ts.value.loc[ts.time == time] = value
         else:
-            ts.value.loc[ts.time == time] = pd.DataFrame(value)
+            ts.value.loc[ts.time == time, :] = np.array(value, dtype=float)
 
     def get_last_n(self, N: int) -> AnomalyResponse:
         """
         returns the response for the last N days
         """
+        cb = self.confidence_band
+        pts = self.predicted_ts
+        ssts = self.stat_sig_ts
 
         return AnomalyResponse(
             scores=self.scores[-N:],
-            confidence_band=ConfidenceBand(
-                upper=self.confidence_band.upper[-N:],
-                lower=self.confidence_band.lower[-N:],
+            confidence_band=None
+            if cb is None
+            else ConfidenceBand(
+                upper=cb.upper[-N:],
+                lower=cb.lower[-N:],
             ),
-            predicted_ts=self.predicted_ts[-N:],
+            predicted_ts=None if pts is None else pts[-N:],
             anomaly_magnitude_ts=self.anomaly_magnitude_ts[-N:],
-            stat_sig_ts=self.stat_sig_ts[-N:],
+            stat_sig_ts=None if ssts is None else ssts[-N:],
         )
 
     def __str__(self) -> str:
+        cb = self.confidence_band
+        upper = None if cb is None else cb.upper.value.values
+        lower = None if cb is None else cb.lower.value.values
+        predicted = (
+            None if self.predicted_ts is None else self.predicted_ts.value.values
+        )
+        statsig = None if self.stat_sig_ts is None else self.stat_sig_ts.value.values
         str_ret = f"""
         Time: {self.scores.time.values},
         Scores: {self.scores.value.values},
-        Upper Confidence Bound: {self.confidence_band.upper.value.values},
-        Lower Confidence Bound: {self.confidence_band.lower.value.values},
-        Predicted Time Series: {self.predicted_ts.value.values},
-        stat_sig:{self.stat_sig_ts.value.values}
+        Upper Confidence Bound: {upper},
+        Lower Confidence Bound: {lower},
+        Predicted Time Series: {predicted},
+        stat_sig:{statsig}
         """
 
         return str_ret
