@@ -1,6 +1,5 @@
-#!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
+#
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
@@ -13,20 +12,21 @@ weighted average and median ensembling.
 """
 
 from __future__ import annotations
-from copy import copy
-import logging
-import sys
-import math
-import pandas as pd
-import numpy as np
-import multiprocessing
-from typing import Any, Callable, Dict, Optional, Sequence, Tuple, Union
-from multiprocessing import cpu_count
 
-import kats.models.model as mm
+import logging
+import math
+import multiprocessing
+import sys
+from copy import copy
+from multiprocessing import cpu_count
+from typing import Any, Callable, Dict, Optional, Sequence, Tuple, Type, Union
+
+import numpy as np
+import pandas as pd
 from kats.consts import TimeSeriesData, Params
-from kats.models.ensemble.ensemble import EnsembleParams
-from kats.models.model import Model
+
+# Seasonality detector
+from kats.detectors.seasonality import ACFDetector
 from kats.models import (
     arima,
     holtwinters,
@@ -36,11 +36,10 @@ from kats.models import (
     sarima,
     theta,
 )
+from kats.models.ensemble.ensemble import EnsembleParams
+from kats.models.model import Model
 from kats.utils.backtesters import BackTesterSimple
 
-# Seasonality detector
-from kats.detectors.seasonality import ACFDetector
-
 # STL decomposition
 from kats.utils.decomposition import TimeSeriesDecomposition
 
@@ -72,24 +71,26 @@ def _logged_error(msg: str) -> ValueError:
     return ValueError(msg)
 
 
-class KatsEnsemble:
+# pyre-fixme[24]: Generic type `Model` expects 1 type parameter.
+class KatsEnsemble(Model):
     """Decomposition based ensemble model in Kats
     This is the holistic ensembling class based on decomposition when seasonality presents
     """
+
     seasonality: bool = False
     sea_data: Optional[TimeSeriesData] = None
     desea_data: Optional[TimeSeriesData] = None
     steps: int = -1
     decomposition_method: str = ""
     model_params: Optional[EnsembleParams] = None
-    fitted: Optional[Dict[Any, Any]] = None
+    fitted: Optional[Dict[str, Any]] = None
     weights: Optional[Dict[str, float]] = None
     predicted: Optional[Dict[str, pd.DataFrame]] = None
     err: Optional[Dict[str, float]] = None
     dates: Optional[pd.DatetimeIndex] = None
     fcst_dates: Optional[ArrayLike] = None
     fcst_df: Optional[pd.DataFrame] = None
-    errors: Optional[Dict[Any, Any]] = None
+    errors: Optional[Dict[str, Any]] = None
 
     def __init__(
         self,
@@ -97,11 +98,11 @@ class KatsEnsemble:
         params: Dict[str, Any],
     ) -> None:
         self.data = data
-        self.freq = pd.infer_freq(data.time)
+        self.freq: Optional[str] = pd.infer_freq(data.time)
         self.params = params
         self.validate_params()
 
-    def validate_params(self):
+    def validate_params(self) -> None:
         # validate aggregation method
         if self.params["aggregation"] not in ("median", "weightedavg"):
             method = self.params["aggregation"]
@@ -117,28 +118,31 @@ class KatsEnsemble:
             self.decomposition_method = "additive"
 
         # validate m
-        if (self.params["seasonality_length"] is not None) and\
-    (self.params["seasonality_length"] > int(len(self.data.time) // 2)):
+        if (self.params["seasonality_length"] is not None) and (
+            self.params["seasonality_length"] > int(len(self.data.time) // 2)
+        ):
             msg = "seasonality_length value cannot be larger than"
             "1/2 of the length of give time series"
             raise _logged_error(msg)
 
         # check customized forecastExecutor
-        if ("forecastExecutor" in self.params.keys()) and\
-    (self.params["forecastExecutor"] is not None):
+        if ("forecastExecutor" in self.params.keys()) and (
+            self.params["forecastExecutor"] is not None
+        ):
             msg = "Using customized forecastExecutor from given parameters"
             logging.info(msg)
             self.forecastExecutor = self.params["forecastExecutor"]
 
         # check customized fitExecutor
-        if ("fitExecutor" in self.params.keys()) and\
-    (self.params["fitExecutor"] is not None):
+        if ("fitExecutor" in self.params.keys()) and (
+            self.params["fitExecutor"] is not None
+        ):
             msg = "Using customized fitExecutor from given parameters"
             logging.info(msg)
             self.fitExecutor = self.params["fitExecutor"]
 
     @staticmethod
-    def seasonality_detector(data) -> bool:
+    def seasonality_detector(data: TimeSeriesData) -> bool:
         """Detect seasonalities from given TimeSeriesData
 
         Args:
@@ -154,7 +158,9 @@ class KatsEnsemble:
         return seasonality
 
     @staticmethod
-    def deseasonalize(data: TimeSeriesData, decomposition_method: str) -> Tuple[TimeSeriesData, TimeSeriesData]:
+    def deseasonalize(
+        data: TimeSeriesData, decomposition_method: str
+    ) -> Tuple[TimeSeriesData, TimeSeriesData]:
         """STL decomposition to given TimeSeriesData
 
         Static method to perform decomposition on the input data
@@ -174,11 +180,9 @@ class KatsEnsemble:
         desea_data = copy(data)
 
         if decomposition_method == "additive":
-            desea_data.value = desea_data.value\
-                - decomp["seasonal"].value
+            desea_data.value = desea_data.value - decomp["seasonal"].value
         else:
-            desea_data.value = desea_data.value\
-                / decomp["seasonal"].value
+            desea_data.value = desea_data.value / decomp["seasonal"].value
         return sea_data, desea_data
 
     @staticmethod
@@ -222,20 +226,23 @@ class KatsEnsemble:
                         desea_pred.set_index("time", inplace=True)
 
                     # native C.I calculated from individual model
-                    predicted[model_name] = desea_pred + \
-                        np.tile(
+                    predicted[model_name] = (
+                        desea_pred
+                        + np.tile(
                             np.tile(seasonality_unit, rep)[:steps], [3, 1]
-                    ).transpose()
+                        ).transpose()
+                    )
                 else:
                     # no C.I from individual model
-                    tmp_fcst = desea_pred.fcst\
-                        + np.tile(seasonality_unit, rep)[:steps]
-                    predicted[model_name] = pd.DataFrame({
-                        "time": desea_pred.index,
-                        "fcst": tmp_fcst,
-                        "fcst_lower": np.nan,
-                        "fcst_upper": np.nan,
-                    }).set_index("time")
+                    tmp_fcst = desea_pred.fcst + np.tile(seasonality_unit, rep)[:steps]
+                    predicted[model_name] = pd.DataFrame(
+                        {
+                            "time": desea_pred.index,
+                            "fcst": tmp_fcst,
+                            "fcst_lower": np.nan,
+                            "fcst_upper": np.nan,
+                        }
+                    ).set_index("time")
 
             else:
                 # multiplicative, element-wise multiply
@@ -250,29 +257,33 @@ class KatsEnsemble:
                         desea_pred.set_index("time", inplace=True)
 
                     # native C.I calculated from individual model
-                    predicted[model_name] = desea_pred * \
-                        np.tile(
+                    predicted[model_name] = (
+                        desea_pred
+                        * np.tile(
                             np.tile(seasonality_unit, rep)[:steps], [3, 1]
-                    ).transpose()
+                        ).transpose()
+                    )
                 else:
                     # no C.I from individual model
-                    tmp_fcst = desea_pred.fcst * \
-                        np.tile(seasonality_unit, rep)[:steps]
-                    predicted[model_name] = pd.DataFrame({
-                        "time": desea_pred.index,
-                        "fcst": tmp_fcst,
-                        "fcst_lower": 0,
-                        "fcst_upper": 0,
-                    }).set_index("time")
+                    tmp_fcst = desea_pred.fcst * np.tile(seasonality_unit, rep)[:steps]
+                    predicted[model_name] = pd.DataFrame(
+                        {
+                            "time": desea_pred.index,
+                            "fcst": tmp_fcst,
+                            "fcst_lower": 0,
+                            "fcst_upper": 0,
+                        }
+                    ).set_index("time")
 
         return predicted
 
     def fitExecutor(
-            self,
-            data : TimeSeriesData,
-            models : EnsembleParams,
-            should_auto_backtest: bool = False,
-    ) -> Tuple[Dict[Any, Any], Optional[Dict[str, float]]]:
+        self,
+        data: TimeSeriesData,
+        models: EnsembleParams,
+        should_auto_backtest: bool = False,
+        err_method: str = "mape",
+    ) -> Tuple[Dict[str, Any], Optional[Dict[str, float]]]:
         """callable forecast executor
 
         This is native implementation with Python's multiprocessing
@@ -296,7 +307,9 @@ class KatsEnsemble:
         if num_process < 1:
             num_process = 1
         # pyre-fixme[16]: `SyncManager` has no attribute `Pool`.
-        pool = multiprocessing.Manager().Pool(processes=(num_process), maxtasksperchild=1000)
+        pool = multiprocessing.Manager().Pool(
+            processes=(num_process), maxtasksperchild=1000
+        )
 
         fitted_models = {}
         for model in models.models:
@@ -305,17 +318,18 @@ class KatsEnsemble:
                 args=(
                     data,
                     MODELS[model.model_name.split("_")[0].lower()],
-                    model.model_params),
+                    model.model_params,
+                ),
             )
         pool.close()
         pool.join()
         fitted = {model: res.get() for model, res in fitted_models.items()}
 
         # if auto back testing
-        weights = self.backTestExecutor() if should_auto_backtest else None
+        weights = self.backTestExecutor(err_method) if should_auto_backtest else None
         return fitted, weights
 
-    def fit(self) -> KatsEnsemble:
+    def fit(self, err_method: str = "mape") -> KatsEnsemble:
         """Fit individual forecasting models via calling fitExecutor
 
         This is the fit methdo to fit individual forecasting model
@@ -333,14 +347,14 @@ class KatsEnsemble:
         auto_backtesting = False if self.params["aggregation"] == "median" else True
 
         # check fitExecutor
-        if "fitExecutor" not in self.params.keys():
+        fitExecutor = self.params.get("fitExecutor")
+        if fitExecutor is None:
             fitExecutor = self.fitExecutor
 
         if self.seasonality:
             # STL decomposition
             sea_data, desea_data = KatsEnsemble.deseasonalize(
-                self.data,
-                self.decomposition_method
+                self.data, self.decomposition_method
             )
             self.sea_data = sea_data
             self.desea_data = desea_data
@@ -361,14 +375,18 @@ class KatsEnsemble:
             )
         else:
             # fit models on the original data
-            self.model_params = model_params = EnsembleParams(self.params["models"].models)
+            self.model_params = model_params = EnsembleParams(
+                self.params["models"].models
+            )
             self.fitted, self.weights = fitExecutor(
                 data=self.data,
                 models=model_params,
                 should_auto_backtest=auto_backtesting,
+                err_method=err_method,
             )
         return self
 
+    # pyre-fixme[14]: `predict` overrides method defined in `Model` inconsistently.
     def predict(self, steps: int) -> KatsEnsemble:
         """Predit future for each individual model
 
@@ -414,8 +432,7 @@ class KatsEnsemble:
             self.predicted = predicted
         else:
             predicted = {
-                k: v.predict(self.steps).set_index("time")
-                for k, v in fitted.items()
+                k: v.predict(self.steps).set_index("time") for k, v in fitted.items()
             }
 
             # add dummy C.I if the model doesn't have native C.I
@@ -423,7 +440,7 @@ class KatsEnsemble:
             # its native C.I if user choose weighted average ensemble.
             for k, v in predicted.items():
                 # if predicted df doesn't have fcst_lower and fcst_upper
-                if "fcst_lower" not in v.columns or "fcst_upper" not in v.columns:
+                if not {"fcst_lower", "fcst_upper"}.issubset(v.columns):
                     # add dummy C.I
                     tmp_v = copy(v)
                     tmp_v["fcst_lower"] = np.nan
@@ -432,7 +449,9 @@ class KatsEnsemble:
             self.predicted = predicted
         return self
 
-    def forecast(self, steps: int) -> Tuple[Dict[str, pd.DataFrame], Optional[Dict[str, float]]]:
+    def forecast(
+        self, steps: int
+    ) -> Tuple[Dict[str, pd.DataFrame], Optional[Dict[str, float]]]:
         """Holistic forecast method in Kats ensemble
 
         combine fit and predict methods to produce forecasted results
@@ -460,8 +479,7 @@ class KatsEnsemble:
         if self.seasonality:
             # call forecastExecutor and move to next steps
             sea_data, desea_data = KatsEnsemble.deseasonalize(
-                self.data,
-                self.decomposition_method
+                self.data, self.decomposition_method
             )
             self.sea_data = sea_data
             self.desea_data = desea_data
@@ -511,6 +529,8 @@ class KatsEnsemble:
                 elif extra_error is not None:
                     desea_err.update(extra_error)
                 self.err = forecast_error = desea_err
+            else:
+                forecast_error = None
         else:
             # no seasonality detected
             predicted, forecast_error = self.forecastExecutor(
@@ -551,12 +571,13 @@ class KatsEnsemble:
             self.weights = None
         return predicted, self.weights
 
-    def forecastExecutor(self,
-                         data : TimeSeriesData,
-                         models : EnsembleParams,
-                         steps: int,
-                         should_auto_backtest: bool = False,
-                         ) -> Tuple[Dict[str, pd.DataFrame], Optional[Dict[str, float]]]:
+    def forecastExecutor(
+        self,
+        data: TimeSeriesData,
+        models: EnsembleParams,
+        steps: int,
+        should_auto_backtest: bool = False,
+    ) -> Tuple[Dict[str, pd.DataFrame], Optional[Dict[str, float]]]:
         """Forecast Executor
 
         This is a callable execution function to
@@ -579,7 +600,9 @@ class KatsEnsemble:
         if num_process < 1:
             num_process = 1
         # pyre-fixme[16]: `SyncManager` has no attribute `Pool`.
-        pool = multiprocessing.Manager().Pool(processes=(num_process), maxtasksperchild=1000)
+        pool = multiprocessing.Manager().Pool(
+            processes=(num_process), maxtasksperchild=1000
+        )
 
         fitted_models = {}
         for model in models.models:
@@ -588,7 +611,8 @@ class KatsEnsemble:
                 args=(
                     data,
                     MODELS[model.model_name.split("_")[0].lower()],
-                    model.model_params),
+                    model.model_params,
+                ),
             )
         pool.close()
         pool.join()
@@ -638,26 +662,32 @@ class KatsEnsemble:
         if self.params["aggregation"].lower() == "median":
             # clean up dataframes with C.I as np.nan or zero
             fcsts = self.clean_dummy_CI(fcsts, use_zero=False)
-            self.fcst_df = fcst_df = pd.DataFrame({
-                "time": dates,
-                "fcst": fcsts["fcst"].median(axis=1),
-                "fcst_lower": fcsts["fcst_lower"].median(axis=1),
-                "fcst_upper": fcsts["fcst_upper"].median(axis=1),
-            })
+            self.fcst_df = fcst_df = pd.DataFrame(
+                {
+                    "time": dates,
+                    "fcst": fcsts["fcst"].median(axis=1),
+                    "fcst_lower": fcsts["fcst_lower"].median(axis=1),
+                    "fcst_upper": fcsts["fcst_upper"].median(axis=1),
+                }
+            )
         else:
-            if fcsts["fcst_lower"].isnull().values.any() or\
-               fcsts["fcst_upper"].isnull().values.any():
+            if (
+                fcsts["fcst_lower"].isnull().values.any()
+                or fcsts["fcst_upper"].isnull().values.any()
+            ):
                 msg = "Conf. interval contains NaN, please check individual model."
                 raise _logged_error(msg)
             weights = self.weights
             assert weights is not None
             weights = np.array(list(weights.values()))
-            self.fcst_df = fcst_df = pd.DataFrame({
-                "time": dates,
-                "fcst": fcsts["fcst"].dot(weights),
-                "fcst_lower": fcsts["fcst_lower"].dot(weights),
-                "fcst_upper": fcsts["fcst_upper"].dot(weights),
-            })
+            self.fcst_df = fcst_df = pd.DataFrame(
+                {
+                    "time": dates,
+                    "fcst": fcsts["fcst"].dot(weights),
+                    "fcst_lower": fcsts["fcst_lower"].dot(weights),
+                    "fcst_upper": fcsts["fcst_upper"].dot(weights),
+                }
+            )
 
         logging.debug("Return forecast data: {fcst_df}".format(fcst_df=fcst_df))
         return fcst_df
@@ -685,7 +715,7 @@ class KatsEnsemble:
             fcsts["fcst_upper"] = fcsts["fcst_upper"].replace(0, np.nan)
         return fcsts
 
-    def backTestExecutor(self) -> Dict[str, float]:
+    def backTestExecutor(self, err_method: str) -> Dict[str, float]:
         """wrapper for back test executor
 
         services which use KatsEnsemble need to write their own backtest wrapper
@@ -697,14 +727,17 @@ class KatsEnsemble:
             The dict of backtesting results
         """
 
-        weights, _ = self._backtester_all()
+        weights, _ = self._backtester_all(err_method=err_method)
         return weights
 
-    def _fit_single(self,
-                    data: TimeSeriesData,
-                    model_func: Callable,
-                    model_param: Params
-                    ) -> Model:
+    def _fit_single(
+        self,
+        data: TimeSeriesData,
+        # pyre-fixme[24]: Generic type `Callable` expects 2 type parameters.
+        model_func: Callable,
+        model_param: Params
+        # pyre-fixme[24]: Generic type `Model` expects 1 type parameter.
+    ) -> Model:
         """Private method to fit individual model
 
         Args:
@@ -724,7 +757,8 @@ class KatsEnsemble:
     def _backtester_single(
         self,
         params: Params,
-        model_class,
+        # pyre-fixme[24]: Generic type `Model` expects 1 type parameter.
+        model_class: Type[Model],
         train_percentage: int = 80,
         test_percentage: int = 20,
         err_method: str = "mape",
@@ -749,7 +783,7 @@ class KatsEnsemble:
             params,
             train_percentage,
             test_percentage,
-            model_class
+            model_class,
         )
         bt.run_backtest()
         return bt.get_error_value(err_method)
@@ -775,14 +809,16 @@ class KatsEnsemble:
         if num_process < 1:
             num_process = 1
         # pyre-fixme[16]: `SyncManager` has no attribute `Pool`.
-        pool = multiprocessing.Manager().Pool(processes=(num_process), maxtasksperchild=1000)
+        pool = multiprocessing.Manager().Pool(
+            processes=(num_process), maxtasksperchild=1000
+        )
         backtesters = {}
         for model in model_params.models:
             backtesters[model.model_name] = pool.apply_async(
                 self._backtester_single,
                 args=(
                     model.model_params,
-                    MODELS[model.model_name.split("_")[0].lower()]
+                    MODELS[model.model_name.split("_")[0].lower()],
                 ),
                 kwds={"err_method": err_method},
             )
@@ -790,21 +826,10 @@ class KatsEnsemble:
         pool.join()
         self.errors = errors = {model: res.get() for model, res in backtesters.items()}
         original_weights = {
-            model: 1 / (err + sys.float_info.epsilon)
-            for model, err in errors.items()
+            model: 1 / (err + sys.float_info.epsilon) for model, err in errors.items()
         }
         weights = {
             model: err / sum(original_weights.values())
             for model, err in original_weights.items()
         }
         return weights, errors
-
-    def plot(self) -> None:
-        """plot forecast results
-        """
-        fcst_df = self.fcst_df
-        if fcst_df is None:
-            raise _logged_error("forecast must be called before plot.")
-
-        logging.info("Generating chart for forecast result from Ensemble model.")
-        mm.Model.plot(self.data, fcst_df)
