@@ -1,9 +1,10 @@
-#!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
+#
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+# pyre-unsafe
+
 """A module for meta-learner hyper-parameter selection.
 
 This module contains two classes, including:
@@ -13,7 +14,7 @@ This module contains two classes, including:
 
 import collections
 import logging
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
 
 import joblib
 import matplotlib.pyplot as plt
@@ -56,7 +57,7 @@ default_model_params = {
     "statsig": {
         "categorical_idx": [],
         "numerical_idx": ["n_control", "n_test"],
-    }
+    },
 }
 
 default_model_networks = {
@@ -276,7 +277,9 @@ class MetaLearnHPT:
         return res
 
     @staticmethod
-    def _get_hidden_and_output_num(n_hidden_num: List[int], out_dim_num: int) -> List[int]:
+    def _get_hidden_and_output_num(
+        n_hidden_num: List[int], out_dim_num: int
+    ) -> List[int]:
         # If there is no numerical variable, out_dim_num = []
         if not out_dim_num:
             return []
@@ -285,7 +288,7 @@ class MetaLearnHPT:
     def build_network(
         self,
         n_hidden_shared: Optional[List[int]] = None,
-        n_hidden_cat_combo: Optional[List[int]] = None,
+        n_hidden_cat_combo: Optional[List[List[int]]] = None,
         n_hidden_num: Optional[List[int]] = None,
     ) -> None:
         """Build a multi-task neural network.
@@ -392,7 +395,7 @@ class MetaLearnHPT:
             else None
         )
         y_num = (
-            torch.from_numpy(self._target_num[train_idx, :].astype('float')).float()
+            torch.from_numpy(self._target_num[train_idx, :].astype("float")).float()
             if self.numerical_idx
             else None
         )
@@ -405,7 +408,7 @@ class MetaLearnHPT:
             else None
         )
         y_num_val = (
-            torch.from_numpy(self._target_num[val_idx, :].astype('float')).float()
+            torch.from_numpy(self._target_num[val_idx, :].astype("float")).float()
             if self.numerical_idx
             else None
         )
@@ -477,7 +480,9 @@ class MetaLearnHPT:
         elif method == "Adam":
             optimizer = torch.optim.Adam(self.model.parameters(), lr=lr)
         else:
-            raise _log_error("Only support SGD and Adam optimaizer. Please use 'SGD' or 'Adam'.")
+            raise _log_error(
+                "Only support SGD and Adam optimaizer. Please use 'SGD' or 'Adam'."
+            )
 
         if val_size <= 0 or val_size >= 1:
             raise _log_error("Illegal validation size.")
@@ -489,7 +494,9 @@ class MetaLearnHPT:
 
         # validate batch size
         if batch_size >= x_fs.size()[0]:
-            raise _log_error(f"batch_size {batch_size} is larger than training data size {x_fs.size()[0]}!")
+            raise _log_error(
+                f"batch_size {batch_size} is larger than training data size {x_fs.size()[0]}!"
+            )
 
         # variables for early stopping
         min_val_loss = np.inf
@@ -555,7 +562,9 @@ class MetaLearnHPT:
         ts = TimeSeriesData(pd.DataFrame(source_ts.to_dataframe().copy()))
 
         if self.model is None:
-            raise _log_error("Haven't trained a model. Please train a model or load a model before predicting.")
+            raise _log_error(
+                "Haven't trained a model. Please train a model or load a model before predicting."
+            )
 
         if ts_scale:
             # scale time series to make ts features more stable
@@ -565,6 +574,7 @@ class MetaLearnHPT:
 
         self.model.eval()
         new_feature = TsFeatures().transform(ts)
+        # pyre-fixme[16]: `List` has no attribute `values`.
         new_feature_vector = np.asarray(list(new_feature.values()))
 
         if np.any(np.isnan(new_feature_vector)):
@@ -602,7 +612,9 @@ class MetaLearnHPT:
         """
 
         if self.model is None:
-            raise _log_error("Haven't trained a model. Please train a model or load a model before predicting.")
+            raise _log_error(
+                "Haven't trained a model. Please train a model or load a model before predicting."
+            )
         if isinstance(source_x, List):
             x = np.row_stack(source_x)
         elif isinstance(source_x, pd.DataFrame):
@@ -667,10 +679,41 @@ class MetaLearnHPT:
         try:
             self.__dict__ = joblib.load(file_path)
         except Exception as e:
-            raise _log_error(f"Fail to load model from {file_path}, and error message is: {e}")
+            raise _log_error(
+                f"Fail to load model from {file_path}, and error message is: {e}"
+            )
 
-    def plot(self):
-        """Plot loss paths of classification/regression on both training and validation set."""
+    def plot(
+        self,
+        figsize: Optional[Tuple[int, int]] = None,
+        xlabel: str = "Epoch",
+        train_linestyle: str = ".-",
+        validation_linestyle: str = "o-",
+        classification_title: str = "Loss path of classification tasks",
+        regression_title: str = "Loss path of regression task",
+        classification_ylabel: str = "Cross-entropy",
+        regression_ylabel: str = "MSE",
+        use_legend: bool = True,
+        legend: Optional[List[str]] = None,
+    ) -> Sequence[plt.Axes]:
+        """Plot loss paths of classification/regression on both training and validation.
+
+        Args:
+            figsize: optional figure size if creating. If None, uses (15, 7).
+            xlabel: x-axis label.
+            train_linestyle: line style to use for training.
+            validation_linestyle: line style to use for validation.
+            classification_title: title for classification plot.
+            regression_title: title for regression plot.
+            classification_ylabel: y-axis label for classification plot.
+            regression_ylabel: y-axis label for regression plot.
+            use_legend: if True, plots legend on both plots.
+            legend: Legend labels to use. if None, uses ["training set",
+                "validation set"].
+
+        Returns:
+            The matplotlib Axes.
+        """
 
         if (
             not self._loss_path["LOSS_train_cat"]
@@ -678,22 +721,29 @@ class MetaLearnHPT:
         ):
             raise _log_error("Using a loaded model or no trained model!")
 
-        plt.figure(figsize=(15, 7))
-        plt.subplot(1, 2, 1)
-        plt.plot(self._loss_path["LOSS_train_cat"], ".-")
-        plt.plot(self._loss_path["LOSS_val_cat"], "o-")
-        plt.legend(["training set", "validation set"])
-        plt.title("Loss path of classification tasks")
-        plt.ylabel("Cross-entropy")
-        plt.xlabel("Epoch")
+        if figsize is None:
+            figsize = (15, 7)
+        _, axs = plt.subplots(1, 2, figsize=figsize, sharex=True)
+        if legend is None:
+            legend = ["training set", "validation set"]
 
-        plt.subplot(1, 2, 2)
-        plt.plot(self._loss_path["LOSS_train_num"], ".-")
-        plt.plot(self._loss_path["LOSS_val_num"], "o-")
-        plt.legend(["training set", "validation set"])
-        plt.title("Loss path of regression task")
-        plt.ylabel("MSE")
-        plt.xlabel("Epoch")
+        ax = axs[0]
+        ax.plot(self._loss_path["LOSS_train_cat"], train_linestyle)
+        ax.plot(self._loss_path["LOSS_val_cat"], validation_linestyle)
+        if use_legend:
+            ax.legend(legend)
+        ax.set_title(classification_title)
+        ax.set_ylabel(classification_ylabel)
+
+        ax = axs[1]
+        ax.plot(self._loss_path["LOSS_train_num"], train_linestyle)
+        ax.plot(self._loss_path["LOSS_val_num"], validation_linestyle)
+        if use_legend:
+            ax.legend(legend)
+        ax.set_title(regression_title)
+        ax.set_ylabel(regression_ylabel)
+        ax.set_xlabel(xlabel)
+        return axs
 
 
 class MultitaskNet(nn.Module):
