@@ -1,19 +1,28 @@
-#!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
+#
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
 import logging
-from typing import Dict, List, Optional
+from typing import Any, Dict, List, Optional
 
-import kats.models.model as m
 import pandas as pd
-from fbprophet import Prophet
+
+try:
+    from fbprophet import Prophet
+
+    _no_prophet = False
+except ImportError:
+    _no_prophet = True
+    Prophet = Dict[str, Any]  # for Pyre
+
 from kats.consts import Params, TimeSeriesData
+from kats.models.model import Model
 from kats.utils.parameter_tuning_utils import (
     get_default_prophet_parameter_search_space,
 )
+
+
 class ProphetParams(Params):
     """Parameter class for Prophet model
 
@@ -70,28 +79,54 @@ class ProphetParams(Params):
         floor: floor, the fcst value must be greater than the specified floor
         custom_seasonlities: customized seasonalities, dict with keys
             "name", "period", "fourier_order"
+        extra_regressors: additional regressors used for fitting, each regressor
+            is a dict with keys "name" and "value"
     """
+
+    growth: str
+    changepoints: Optional[List[float]]
+    n_changepoints: int
+    changepoint_range: float
+    yearly_seasonality: str
+    weekly_seasonality: str
+    daily_seasonality: str
+    holidays: Optional[pd.DataFrame]
+    seasonality_mode: str
+    seasonality_prior_scale: float
+    holidays_prior_scale: float
+    changepoint_prior_scale: float
+    mcmc_samples: int
+    interval_width: float
+    uncertainty_samples: int
+    cap: Optional[float]
+    floor: Optional[float]
+    custom_seasonalities: List[Dict[str, Any]]
+    extra_regressors: List[Dict[str, Any]]
+
     def __init__(
         self,
-        growth="linear",
-        changepoints=None,
-        n_changepoints=25,
-        changepoint_range=0.8,
-        yearly_seasonality="auto",
-        weekly_seasonality="auto",
-        daily_seasonality="auto",
-        holidays=None,
-        seasonality_mode="additive",
-        seasonality_prior_scale=10.0,
-        holidays_prior_scale=10.0,
-        changepoint_prior_scale=0.05,
-        mcmc_samples=0,
-        interval_width=0.80,
-        uncertainty_samples=1000,
-        cap=None,
-        floor=None,
-        custom_seasonalities: Optional[List[Dict]] = None,
+        growth: str = "linear",
+        changepoints: Optional[List[float]] = None,
+        n_changepoints: int = 25,
+        changepoint_range: float = 0.8,
+        yearly_seasonality: str = "auto",
+        weekly_seasonality: str = "auto",
+        daily_seasonality: str = "auto",
+        holidays: Optional[pd.DataFrame] = None,
+        seasonality_mode: str = "additive",
+        seasonality_prior_scale: float = 10.0,
+        holidays_prior_scale: float = 10.0,
+        changepoint_prior_scale: float = 0.05,
+        mcmc_samples: int = 0,
+        interval_width: float = 0.80,
+        uncertainty_samples: int = 1000,
+        cap: Optional[float] = None,
+        floor: Optional[float] = None,
+        custom_seasonalities: Optional[List[Dict[str, Any]]] = None,
+        extra_regressors: Optional[List[Dict[str, Any]]] = None,
     ) -> None:
+        if _no_prophet:
+            raise RuntimeError("requires fbprophet to be installed")
         super().__init__()
         self.growth = growth
         self.changepoints = changepoints
@@ -111,9 +146,9 @@ class ProphetParams(Params):
         self.cap = cap
         self.floor = floor
         self.custom_seasonalities = (
-            [] if custom_seasonalities is None
-            else custom_seasonalities
+            [] if custom_seasonalities is None else custom_seasonalities
         )
+        self.extra_regressors = [] if extra_regressors is None else extra_regressors
         logging.debug(
             "Initialized Prophet with parameters. "
             "growth:{growth},"
@@ -133,7 +168,8 @@ class ProphetParams(Params):
             "uncertainty_samples:{uncertainty_samples},"
             "cap:{cap},"
             "floor:{floor},"
-            "custom_seasonalities:{custom_seasonalities}".format(
+            "custom_seasonalities:{custom_seasonalities},"
+            "extra_regressors:{extra_regressors}".format(
                 growth=growth,
                 changepoints=changepoints,
                 n_changepoints=n_changepoints,
@@ -152,10 +188,13 @@ class ProphetParams(Params):
                 cap=cap,
                 floor=floor,
                 custom_seasonalities=custom_seasonalities,
+                extra_regressors=None
+                if extra_regressors is None
+                else [x["name"] for x in extra_regressors],
             )
         )
 
-    def validate_params(self):
+    def validate_params(self) -> None:
         """validate Prophet parameters
 
         This method validates some key parameters including growth rate
@@ -178,11 +217,22 @@ class ProphetParams(Params):
             logging.error(msg)
             raise ValueError(msg)
 
+        # If extra_regressors passed, ensure they contain the required keys.
+        reqd_regressor_keys = ["name", "value"]
+        if not all(
+            req_key in regressor
+            for req_key in reqd_regressor_keys
+            for regressor in self.extra_regressors
+        ):
+            msg = f"Extra regressor dicts must contain the following keys:\n{reqd_regressor_keys}"
+            logging.error(msg)
+            raise ValueError(msg)
+
         logging.info("Method validate_params() is not fully implemented.")
         pass
 
 
-class ProphetModel(m.Model):
+class ProphetModel(Model[ProphetParams]):
     """Model class for Prophet
 
     This class provides fit, predict, and plot methods for Prophet model
@@ -191,15 +241,22 @@ class ProphetModel(m.Model):
         data: the input time series data as in :class:`kats.consts.TimeSeriesData`
         params: the parameter class definied with `ProphetParams`
     """
+
+    model: Optional[Prophet] = None
+    freq: Optional[str] = None
+
     def __init__(self, data: TimeSeriesData, params: ProphetParams) -> None:
         super().__init__(data, params)
+        if _no_prophet:
+            raise RuntimeError("requires fbprophet to be installed")
         if not isinstance(self.data.value, pd.Series):
             msg = "Only support univariate time series, but get {type}.".format(
                 type=type(self.data.value)
             )
             logging.error(msg)
             raise ValueError(msg)
-    def fit(self, **kwargs) -> None:
+
+    def fit(self, **kwargs: Any) -> None:
         """fit Prophet model
 
         Args:
@@ -229,7 +286,8 @@ class ProphetModel(m.Model):
             "uncertainty_samples:{uncertainty_samples},"
             "cap:{cap},"
             "floor:{floor},"
-            "custom_seasonalities:{custom_seasonalities}".format(
+            "custom_seasonalities:{custom_seasonalities},"
+            "extra_regressors:{extra_regressors}".format(
                 growth=self.params.growth,
                 changepoints=self.params.changepoints,
                 n_changepoints=self.params.n_changepoints,
@@ -248,7 +306,10 @@ class ProphetModel(m.Model):
                 cap=self.params.cap,
                 floor=self.params.floor,
                 custom_seasonalities=self.params.custom_seasonalities,
-            )
+                extra_regressors=None
+                if self.params.extra_regressors is None
+                else [x["name"] for x in self.params.extra_regressors],
+            ),
         )
 
         prophet = Prophet(
@@ -281,12 +342,20 @@ class ProphetModel(m.Model):
         for custom_seasonality in self.params.custom_seasonalities:
             prophet.add_seasonality(**custom_seasonality)
 
-        # pyre-fixme[16]: `ProphetModel` has no attribute `model`.
+        # Add any extra regressors
+        if self.params.extra_regressors is not None:
+            for regressor in self.params.extra_regressors:
+                prophet.add_regressor(
+                    **{k: v for k, v in regressor.items() if k not in ["value"]}
+                )
+                df[regressor["name"]] = pd.Series(regressor["value"], index=df.index)
+
         self.model = prophet.fit(df=df)
         logging.info("Fitted Prophet model. ")
 
-    # pyre-fixme[14]: `predict` overrides method defined in `Model` inconsistently.
-    def predict(self, steps, include_history=False, **kwargs) -> pd.DataFrame:
+    def predict(
+        self, steps: int, *args: Any, include_history: bool = False, **kwargs: Any
+    ) -> pd.DataFrame:
         """predict with fitted Prophet model
 
         Args:
@@ -297,42 +366,61 @@ class ProphetModel(m.Model):
             The predicted dataframe with following columns:
                 `time`, `fcst`, `fcst_lower`, and `fcst_upper`
         """
+        model = self.model
+        if model is None:
+            raise ValueError("Call fit() before predict().")
+
         logging.debug(
             "Call predict() with parameters. "
             "steps:{steps}, kwargs:{kwargs}".format(steps=steps, kwargs=kwargs)
         )
-        # pyre-fixme[16]: `ProphetModel` has no attribute `freq`.
-        # pyre-fixme[16]: `ProphetModel` has no attribute `data`.
         self.freq = kwargs.get("freq", pd.infer_freq(self.data.time))
-        # pyre-fixme[16]: `ProphetModel` has no attribute `include_history`.
         self.include_history = include_history
         # prepare future for Prophet.predict
         future = kwargs.get("future")
         raw = kwargs.get("raw", False)
         if future is None:
-            # pyre-fixme[16]: `ProphetModel` has no attribute `model`.
-            # pyre-fixme[16]: `Params` has no attribute `cap`.
-            future = self.model.make_future_dataframe(
-                periods=steps,
-                freq=self.freq,
-                include_history=self.include_history)
+            future = model.make_future_dataframe(
+                periods=steps, freq=self.freq, include_history=self.include_history
+            )
             if self.params.growth == "logistic":
                 # assign cap to a new col as Prophet required
                 future["cap"] = self.params.cap
             if self.params.floor is not None:
                 future["floor"] = self.params.floor
 
-        fcst = self.model.predict(future).tail(steps)
+        extra_regressors = kwargs.get("extra_regressors", None)
+        if extra_regressors is not None:
+            for regressor in extra_regressors:
+
+                if not self.include_history:
+                    future[regressor["name"]] = pd.Series(
+                        regressor["value"], index=future.index
+                    )
+                    continue
+
+                # If history is required, it has to be pulled back from the
+                # parameter object and combined with future values
+                regressor_item = filter(
+                    lambda x: x["name"] == regressor["name"],
+                    self.params.extra_regressors,
+                ).__next__()
+
+                regressor_value = pd.concat(
+                    [pd.Series(regressor["value"]), pd.Series(regressor_item["value"])],
+                    ignore_index=True,
+                )
+                regressor_value.index = future.index
+                future[regressor["name"]] = regressor_value
+
+        fcst = model.predict(future)
         if raw:
-            return fcst
+            return fcst.tail(steps)
 
-        # if include_history:
-        fcst = self.model.predict(future)
         logging.info("Generated forecast data from Prophet model.")
         logging.debug("Forecast data: {fcst}".format(fcst=fcst))
 
-        # pyre-fixme[16]: `ProphetModel` has no attribute `fcst_df`.
-        self.fcst_df = pd.DataFrame(
+        self.fcst_df = fcst_df = pd.DataFrame(
             {
                 "time": fcst.ds,
                 "fcst": fcst.yhat,
@@ -342,21 +430,12 @@ class ProphetModel(m.Model):
         )
 
         logging.debug("Return forecast data: {fcst_df}".format(fcst_df=self.fcst_df))
-        return self.fcst_df
+        return fcst_df
 
-    def plot(self):
-        """plot forecasted results from Prophet model
-        """
-        logging.info("Generating chart for forecast result from Prophet model.")
-        m.Model.plot(self.data, self.fcst_df, include_history=self.include_history)
-
-    def __str__(self):
+    def __str__(self) -> str:
         return "Prophet"
 
     @staticmethod
     def get_parameter_search_space() -> List[Dict[str, object]]:
-        """get default parameter search space for Prophet model
-        """
-        # pyre-fixme[7]: Expected `List[Dict[str, object]]` but got `List[Dict[str,
-        #  typing.Union[List[typing.Any], bool, str]]]`.
+        """get default parameter search space for Prophet model"""
         return get_default_prophet_parameter_search_space()
