@@ -1,6 +1,5 @@
-#!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
+#
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
@@ -24,11 +23,13 @@ For more information, check out the Kats tutorial notebook on backtesting!
 import logging
 import multiprocessing as mp
 from abc import ABC, abstractmethod
-from typing import List, Optional, Tuple, Type
+from typing import Any, Callable, Dict, List, Optional, Tuple, Type, TYPE_CHECKING
 
 import numpy as np
 import pandas as pd
 from kats.consts import Params, TimeSeriesData
+if TYPE_CHECKING:
+    from kats.models.model import Model
 
 
 # Constant to indicate error types supported
@@ -59,17 +60,33 @@ class BackTesterParent(ABC):
     Raises:
       ValueError: The time series is empty or an invalid error type was passed.
     """
+    error_methods: List[str]
+    data: TimeSeriesData
+    # pyre-fixme[24]: Generic type `type` expects 1 type parameter, use
+    #  `typing.Type` to avoid runtime subscripting errors.
+    model_class: Type
+    params: Params
+    multi: bool
+    offset: int
+    results: List[Tuple[np.ndarray, np.ndarray, 'Model[Params]', np.ndarray]]
+    errors: Dict[str, float]
+    size: int
+    error_funcs: Dict[str, Callable[[np.ndarray, np.ndarray, np.ndarray, np.ndarray], float]]
+    freq: Optional[str]
+    raw_errors: List[np.ndarray]
 
     def __init__(
         self,
         error_methods: List[str],
         data: TimeSeriesData,
         params: Params,
+        # pyre-fixme[24]: Generic type `type` expects 1 type parameter, use
+        #  `typing.Type` to avoid runtime subscripting errors.
         model_class: Type,
         multi: bool,
-        offset=0,
-        **kwargs
-    ):
+        offset: int = 0,
+        **kwargs: Any,
+    ) -> None:
         self.error_methods = error_methods
         self.data = data
         self.model_class = model_class
@@ -225,17 +242,15 @@ class BackTesterParent(ABC):
         truth: np.ndarray,
         diffs: np.ndarray,
     ) -> float:
-        """Calculates MASE error.
+        """Calculates non-seasonal MASE error.
 
         mean(|actual - forecast| / naiveError), where
-        naiveError = 1/ (n-1) sigma^n_[i=2](|actual_[i] - actual_[i-1]|)
+        naiveError = mean(|actual_[i] - actual_[i-1]|)
         """
 
         logging.info("Calculating MASE")
-        naive_error = np.abs(np.diff(training_inputs)).sum() / (
-            training_inputs.shape[0] - 1
-        )
-        return diffs.mean() / naive_error
+        naive_errors = np.abs(np.diff(training_inputs))
+        return diffs.mean() / naive_errors.mean()
 
     def _calc_mse(
         self,
@@ -265,7 +280,7 @@ class BackTesterParent(ABC):
         self,
         training_data_indices: Tuple[int, int],
         testing_data_indices: Tuple[int, int],
-    ) -> Optional[Tuple[np.ndarray, np.ndarray, Type, np.ndarray]]:
+    ) -> Optional[Tuple[np.ndarray, np.ndarray, 'Model[Params]', np.ndarray]]:
         """
         Trains model, evaluates it, and stores results in results list.
         """
@@ -365,7 +380,11 @@ class BackTesterParent(ABC):
                 pool.apply_async(self._create_model, args=(train_split, test_split))
                 for train_split, test_split in zip(training_splits, testing_splits)
             ]
-            self.results = [fut.get() for fut in futures]
+            self.results = results = []
+            for fut in futures:
+                result = fut.get()
+                assert result is not None
+                results.append(result)
             pool.close()
 
     def run_backtest(self) -> None:
@@ -394,8 +413,8 @@ class BackTesterParent(ABC):
             raise ValueError("Invalid error name")
 
     @abstractmethod
-    def _create_train_test_splits(self):
-        pass
+    def _create_train_test_splits(self) -> Tuple[List[Tuple[int, int]], List[Tuple[int, int]]]:
+        raise NotImplementedError()
 
 
 class BackTesterSimple(BackTesterParent):
@@ -448,9 +467,11 @@ class BackTesterSimple(BackTesterParent):
         params: Params,
         train_percentage: float,
         test_percentage: float,
+        # pyre-fixme[24]: Generic type `type` expects 1 type parameter, use
+        #  `typing.Type` to avoid runtime subscripting errors.
         model_class: Type,
-        **kwargs
-    ):
+        **kwargs: Any,
+    ) -> None:
         logging.info("Initializing train/test percentages")
         if train_percentage <= 0:
             logging.error("Non positive training percentage")
@@ -476,8 +497,8 @@ class BackTesterSimple(BackTesterParent):
         """Creates train/test folds for the backtest."""
 
         logging.info("Creating train test splits")
-        train_size = _get_percent_size(self.size, self.train_percentage)
-        test_size = _get_percent_size(self.size, self.test_percentage)
+        train_size = _get_absolute_size(self.size, self.train_percentage)
+        test_size = _get_absolute_size(self.size, self.test_percentage)
 
         if train_size <= 0 or train_size >= self.size:
             logging.error("Invalid training size: {0}".format(train_size))
@@ -500,23 +521,34 @@ class BackTesterSimple(BackTesterParent):
         return [(0, train_size)], [(train_size, train_size + test_size)]
 
 
-class BackTesterExpandingWindow(BackTesterParent):
-    """Defines functions to execute an expanding window backtest.
+class BackTesterRollingOrigin(BackTesterParent):
+    """Defines functions to execute an rolling origin backtest.
 
-    An expanding window backtest conducts a backtest over multiple iterations,
-    wherein each iteration, the size of the training dataset increases by a
-    fixed amount, while the test dataset "slides" forward to accommodate.
+    A rolling forecast origin backtest conducts a backtest over multiple
+    iterations, wherein each iteration, the "forecasting origin"
+    (the location separating training and testing datasets) "slides" forward
+    by a fixed amount.
+
+    Hereby, the size of the training dataset is usually increased at each
+    iteration, while the test dataset "slides" forward to accommodate.
+    However, the size of the training dataset can alternatively be held
+    constant, in which case at each iteration the start location of the
+    training dataset moves forward by the same amount as the "forecast origin".
     Iterations continue until the complete data set is used to either train
     or test in the final interation.
 
+    This procedure is also known in literature as a rolling origin evaluation
+    with a continuously increasing in-sample size (train set) and a constant
+    out-sample size (test set).
     For more information, check out the Kats tutorial notebooks!
 
     Attributes:
       start_train_percentage: A float for the initial percentage of data used
-        for training.
-      end_train_percentage: A float for the final percentage of data used for
-        training.
+        for training. (The train percentage at the end will be 100 -
+        test_percentage)
       test_percentage: A float for the percentage of data used for testing.
+        (The test set is taken at sliding positions from start_train_percentage
+         up to the end of the dataset - only the last fold is at the very end.)
       expanding_steps: An integer for the number of expanding steps (i.e.
         number of folds).
       error_methods: List of strings indicating which errors to calculate
@@ -524,6 +556,8 @@ class BackTesterExpandingWindow(BackTesterParent):
       data: :class:`kats.consts.TimeSeriesData` object to perform backtest on.
       params: Parameters to train model with.
       model_class: Defines the model type to use for backtesting.
+      constant_train_size: A bool defining if the training data size should be
+        constant instead of expanding it at each iteration (default False).
       multi: A boolean flag to toggle multiprocessing (default True).
       results: List of tuples `(training_data, testing_data, trained_model,
         forecast_predictions)` storing forecast results.
@@ -544,15 +578,15 @@ class BackTesterExpandingWindow(BackTesterParent):
       >>> ts = TimeSeriesData(df=df)
       >>> params = ARIMAParams(p=1, d=1, q=1)
       >>> all_errors = ["mape", "smape", "mae", "mase", "mse", "rmse"]
-      >>> backtester = BackTesterExpandingWindow(
+      >>> backtester = BackTesterRollingOrigin(
             error_methods=all_errors,
             data=ts,
-            params=paramsparams,
+            params=params,
             start_train_percentage=50,
-            end_train_percentage=75,
             test_percentage=25,
             expanding_steps=3,
             model_class=ARIMAModel,
+            constant_train_size=False,
           )
       >>> backtester.run_backtest()
       >>> mape = backtester.get_error_value("mape") # Retrieve MAPE error
@@ -564,13 +598,15 @@ class BackTesterExpandingWindow(BackTesterParent):
         data: TimeSeriesData,
         params: Params,
         start_train_percentage: float,
-        end_train_percentage: float,
         test_percentage: float,
         expanding_steps: int,
+        # pyre-fixme[24]: Generic type `type` expects 1 type parameter, use
+        #  `typing.Type` to avoid runtime subscripting errors.
         model_class: Type,
-        multi=True,
-        **kwargs
-    ):
+        constant_train_size: bool = False,
+        multi: bool=True,
+        **kwargs: Any,
+    ) -> None:
         logging.info("Initializing train/test percentages")
         if start_train_percentage <= 0:
             logging.error("Non positive start training percentage")
@@ -579,16 +615,6 @@ class BackTesterExpandingWindow(BackTesterParent):
             logging.error("Too large start training percentage")
             raise ValueError("Invalid end training percentage")
         self.start_train_percentage = start_train_percentage
-        if end_train_percentage <= 0:
-            logging.error("Non positive end training percentage")
-            raise ValueError("Invalid start training percentage")
-        elif end_train_percentage > 100:
-            logging.error("Too large end training percentage")
-            raise ValueError("Invalid end training percentage")
-        elif end_train_percentage < self.start_train_percentage:
-            logging.error("Ending Training % < Start Training %")
-            raise ValueError("Start Training percentage must be less than End")
-        self.end_train_percentage = end_train_percentage
         if test_percentage <= 0:
             logging.error("Non positive test percentage")
             raise ValueError("Invalid test percentage")
@@ -596,10 +622,27 @@ class BackTesterExpandingWindow(BackTesterParent):
             logging.error("Too large test percentage")
             raise ValueError("Invalid test percentage")
         self.test_percentage = test_percentage
+        if start_train_percentage + test_percentage > 100:
+            logging.error("Too large combined train and test percentage")
+            raise ValueError(  # noqa
+                "Invalid training and testing percentage combination."
+            )
+        elif start_train_percentage + test_percentage == 100:
+            if expanding_steps > 1:
+                logging.error(
+                    "Too large combined train and test percentage for "
+                    "%s expanding steps.",
+                    expanding_steps,
+                )
+                raise ValueError(
+                    "Invalid trraining and testing percentage combination "
+                    f"given for {expanding_steps} expanding steps"
+                )
         if expanding_steps < 0:
             logging.error("Non positive expanding steps")
             raise ValueError("Invalid expanding steps")
         self.expanding_steps = expanding_steps
+        self.constant_train_size = constant_train_size
 
         logging.info("Calling parent class constructor")
         super().__init__(error_methods, data, params, model_class, multi, **kwargs)
@@ -610,9 +653,8 @@ class BackTesterExpandingWindow(BackTesterParent):
         """Creates train/test folds for the backtest."""
 
         logging.info("Creating train test splits")
-        start_train_size = _get_percent_size(self.size, self.start_train_percentage)
-        end_train_size = _get_percent_size(self.size, self.end_train_percentage)
-        test_size = _get_percent_size(self.size, self.test_percentage)
+        start_train_size = _get_absolute_size(self.size, self.start_train_percentage)
+        test_size = _get_absolute_size(self.size, self.test_percentage)
 
         if start_train_size <= 0 or start_train_size >= self.size:
             logging.error(
@@ -623,30 +665,36 @@ class BackTesterExpandingWindow(BackTesterParent):
             )
             raise ValueError("Incorrect starting training size")
 
-        if end_train_size <= 0 or end_train_size >= self.size:
-            logging.error("Invalid ending training size: {0}".format(end_train_size))
-            logging.error(
-                "End Training Percentage: {0}".format(self.end_train_percentage)
-            )
-            logging.error(
-                "End Training Percentage: {0}".format(self.end_train_percentage)
-            )
-            raise ValueError("Incorrect starting training size")
-
         if test_size <= 0 or test_size >= self.size:
             logging.error("Invalid testing size: {0}".format(test_size))
             logging.error("Testing Percentage: {0}".format(self.test_percentage))
             raise ValueError("Incorrect testing size")
 
-        if end_train_size + test_size > self.size:
-            logging.error("Training and Testing sizes too big")
-            logging.error("End Training size: {0}".format(end_train_size))
-            logging.error(
-                "End Training Percentage: {0}".format(self.end_train_percentage)
-            )
+        if start_train_size + test_size > self.size:
+            if start_train_size + test_size > self.size:
+                logging.error("Training and Testing sizes too big")
+                logging.error("Start Training size: {0}".format(start_train_size))
+                logging.error(
+                    "Start Training Percentage: {0}".format(self.start_train_percentage)
+                )
             logging.error("Testing size: {0}".format(test_size))
             logging.error("Testing Percentage: {0}".format(self.test_percentage))
             raise ValueError("Incorrect training and testing sizes")
+        elif start_train_size + test_size == self.size:
+            if self.expanding_steps > 1:
+                logging.error(
+                    "Training and Testing sizes too big " "for multiple steps"
+                )
+                logging.error("Start Training size: {0}".format(start_train_size))
+                logging.error(
+                    "Start Training Percentage: {0}".format(self.start_train_percentage)
+                )
+                logging.error("Testing size: {0}".format(test_size))
+                logging.error("Testing Percentage: {0}".format(self.test_percentage))
+                logging.error("Expanding steps: {}".format(self.expanding_steps))
+                raise ValueError(
+                    "Incorrect training and testing sizes " "for multiple steps"
+                )
 
         # Handling edge case where only 1 fold is needed (same as BackTesterSimple)
         if self.expanding_steps == 1:
@@ -658,10 +706,13 @@ class BackTesterExpandingWindow(BackTesterParent):
         train_splits = []
         test_splits = []
         offsets = _return_fold_offsets(
-            start_train_size, end_train_size, self.expanding_steps
+            0, self.size - start_train_size - test_size, self.expanding_steps
         )
         for offset in offsets:
-            train_splits.append((0, int(start_train_size + offset)))
+            skip_size = 0
+            if self.constant_train_size:
+                skip_size = offset
+            train_splits.append((skip_size, int(start_train_size + offset)))
             test_splits.append(
                 (
                     int(start_train_size + offset),
@@ -671,58 +722,51 @@ class BackTesterExpandingWindow(BackTesterParent):
         return train_splits, test_splits
 
 
-class BackTesterRollingWindow(BackTesterParent):
+class BackTesterExpandingWindow(BackTesterRollingOrigin):
+    """Defines functions to exectute an expanding window backtest.
+
+    This class will be deprecated soon. Please use `BackTesterRollingOrigin`
+    with param `constant_train_size = True`.
+    """
+
+    def __init__(
+        self,
+        error_methods: List[str],
+        data: TimeSeriesData,
+        params: Params,
+        start_train_percentage: float,
+        end_train_percentage: float,
+        test_percentage: float,
+        expanding_steps: int,
+        # pyre-fixme[24]: Generic type `type` expects 1 type parameter, use
+        #  `typing.Type` to avoid runtime subscripting errors.
+        model_class: Type,
+        multi: bool=True,
+        **kwargs: Any,
+    ) -> None:
+        logging.info(
+            "BackTesterExpandingWindow will be deprecated. Please use the "
+            "updated API found in BackTesterRollingOrigin."
+        )
+        super().__init__(
+            error_methods=error_methods,
+            data=data,
+            params=params,
+            start_train_percentage=start_train_percentage,
+            test_percentage=test_percentage,
+            expanding_steps=expanding_steps,
+            model_class=model_class,
+            multi=multi,
+            constant_train_size=False,
+            **kwargs,
+        )
+
+
+class BackTesterRollingWindow(BackTesterRollingOrigin):
     """Defines functions to execute a rolling window backtest.
 
-    An rolling window backtest conducts a backtest over multiple iterations,
-    wherein each iteration, the start location of the training dataset moves
-    forward by a fixed amount, while the test dataset "slides" forward to
-    accommodate. Iterations continue until the end of the test set meets the
-    end of the full data set.
-
-    For more information, check out the Kats tutorial notebooks!
-
-    Attributes:
-      train_percentage: A float for the percentage of data used for training.
-      test_percentage: A float for the percentage of data used for testing.
-      sliding_steps: An integer for the number of rolling steps (i.e.
-        number of folds).
-      error_methods: List of strings indicating which errors to calculate
-        (see `ALLOWED_ERRORS` for exhaustive list).
-      data: :class:`kats.consts.TimeSeriesData` object to perform backtest on.
-      params: Parameters to train model with.
-      model_class: Defines the model type to use for backtesting.
-      multi: A boolean flag to toggle multiprocessing (default True).
-      results: List of tuples `(training_data, testing_data, trained_model,
-        forecast_predictions)` storing forecast results.
-      errors: Dictionary mapping the error type to value.
-      size: An integer for the total number of datapoints.
-      error_funcs: Dictionary mapping error name to the
-        function that calculates it.
-      freq: A string representing the (inferred) frequency of the
-        `pandas.DataFrame`.
-      raw_errors: List storing raw errors (truth - predicted).
-
-    Raises:
-      ValueError: One or more of the train, test, or sliding steps params
-        were invalid. Or the time series is empty.
-
-    Sample Usage:
-      >>> df = pd.read_csv("kats/data/air_passengers.csv")
-      >>> ts = TimeSeriesData(df=df)
-      >>> params = ARIMAParams(p=1, d=1, q=1)
-      >>> all_errors = ["mape", "smape", "mae", "mase", "mse", "rmse"]
-      >>> backtester = BackTesterExpandingWindow(
-            error_methods=all_errors,
-            data=ts,
-            params=paramsparams,
-            train_percentage=50,
-            test_percentage=25,
-            expanding_steps=3,
-            model_class=ARIMAModel,
-          )
-      >>> backtester.run_backtest()
-      >>> mape = backtester.get_error_value("mape") # Retrieve MAPE error
+    This class will be deprecated soon. Please use `BackTesterRollingOrigin`
+    with param `constant_train_size = False`.
     """
 
     def __init__(
@@ -733,75 +777,28 @@ class BackTesterRollingWindow(BackTesterParent):
         train_percentage: float,
         test_percentage: float,
         sliding_steps: int,
+        # pyre-fixme[24]: Generic type `type` expects 1 type parameter, use
+        #  `typing.Type` to avoid runtime subscripting errors.
         model_class: Type,
-        multi=True,
-        **kwargs
-    ):
-        logging.info("Initializing train/test percentages")
-        if train_percentage <= 0:
-            logging.error("Non positive training percentage")
-            raise ValueError("Invalid training percentage")
-        elif train_percentage > 100:
-            logging.error("Too large training percentage")
-            raise ValueError("Invalid training percentage")
-        self.train_percentage = train_percentage
-        if test_percentage <= 0:
-            logging.error("Non positive test percentage")
-            raise ValueError("Invalid test percentage")
-        elif test_percentage > 100:
-            logging.error("Too large test percentage")
-            raise ValueError("Invalid test percentage")
-        self.test_percentage = test_percentage
-        if sliding_steps < 0:
-            logging.error("Non positive sliding steps")
-            raise ValueError("Invalid sliding steps")
-        self.sliding_steps = sliding_steps
-
-        logging.info("Calling parent class constructor")
-        super().__init__(error_methods, data, params, model_class, multi, **kwargs)
-
-    def _create_train_test_splits(
-        self,
-    ) -> Tuple[List[Tuple[int, int]], List[Tuple[int, int]]]:
-        """Creates train/test folds for the backtest."""
-
-        logging.info("Creating train test splits")
-        train_size = _get_percent_size(self.size, self.train_percentage)
-        test_size = _get_percent_size(self.size, self.test_percentage)
-
-        if train_size <= 0 or train_size >= self.size:
-            logging.error("Invalid training size: {0}".format(train_size))
-            logging.error("Training Percentage: {0}".format(self.train_percentage))
-            raise ValueError("Incorrect training size")
-
-        if test_size <= 0 or test_size >= self.size:
-            logging.error("Invalid testing size: {0}".format(test_size))
-            logging.error("Testing Percentage: {0}".format(self.test_percentage))
-            raise ValueError("Incorrect testing size")
-
-        if train_size + test_size > self.size:
-            logging.error("Training and Testing sizes too big")
-            logging.error("Training size: {0}".format(train_size))
-            logging.error("Training Percentage: {0}".format(self.train_percentage))
-            logging.error("Testing size: {0}".format(test_size))
-            logging.error("Testing Percentage: {0}".format(self.test_percentage))
-            raise ValueError("Incorrect training and testing sizes")
-
-        # Handling edge case where only 1 fold is needed (same as BackTesterSimple)
-        if self.sliding_steps == 1:
-            return [(0, train_size)], [(train_size, train_size + test_size)]
-
-        train_splits = []
-        test_splits = []
-        offsets = _return_fold_offsets(
-            0, self.size - train_size - test_size, self.sliding_steps
+        multi: bool=True,
+        **kwargs: Any,
+    ) -> None:
+        logging.info(
+            "BackTesterRollingWindow will be deprecated. Please use the "
+            "updated API found in BackTesterRollingOrigin."
+        )
+        super().__init__(
+            error_methods=error_methods,
+            data=data,
+            params=params,
+            start_train_percentage=train_percentage,
+            test_percentage=test_percentage,
+            expanding_steps=sliding_steps,
+            model_class=model_class,
+            multi=multi,
+            constant_train_size=True,
+            **kwargs,
         )
-        for offset in offsets:
-            train_splits.append((offset, int(offset + train_size)))
-            test_splits.append(
-                (int(offset + train_size), int(offset + train_size + test_size))
-            )
-        return train_splits, test_splits
 
 
 class BackTesterFixedWindow(BackTesterParent):
@@ -844,7 +841,7 @@ class BackTesterFixedWindow(BackTesterParent):
       >>> backtester = BackTesterFixedWindow(
             error_methods=all_errors,
             data=ts,
-            params=paramsparams,
+            params=params,
             train_percentage=50,
             test_percentage=25,
             window_percentage=25,
@@ -862,9 +859,11 @@ class BackTesterFixedWindow(BackTesterParent):
         train_percentage: float,
         test_percentage: float,
         window_percentage: int,
+        # pyre-fixme[24]: Generic type `type` expects 1 type parameter, use
+        #  `typing.Type` to avoid runtime subscripting errors.
         model_class: Type,
-        **kwargs
-    ):
+        **kwargs: Any,
+    ) -> None:
         logging.info("Initializing train/test percentages")
         if train_percentage <= 0:
             logging.error("Non positive training percentage")
@@ -888,7 +887,7 @@ class BackTesterFixedWindow(BackTesterParent):
             raise ValueError("Invalid window percentage")
         self.window_percentage = window_percentage
 
-        offset = _get_percent_size(len(data.time), self.window_percentage)
+        offset = _get_absolute_size(len(data.time), self.window_percentage)
 
         logging.info("Calling parent class constructor")
         super().__init__(
@@ -901,9 +900,9 @@ class BackTesterFixedWindow(BackTesterParent):
         """Creates train/test folds for the backtest."""
 
         logging.info("Creating train test splits")
-        train_size = _get_percent_size(self.size, self.train_percentage)
-        test_size = _get_percent_size(self.size, self.test_percentage)
-        window_size = _get_percent_size(self.size, self.window_percentage)
+        train_size = _get_absolute_size(self.size, self.train_percentage)
+        test_size = _get_absolute_size(self.size, self.test_percentage)
+        window_size = _get_absolute_size(self.size, self.window_percentage)
 
         if train_size <= 0 or train_size >= self.size:
             logging.error("Invalid training size: {0}".format(train_size))
@@ -933,13 +932,15 @@ class BackTesterFixedWindow(BackTesterParent):
 
 
 class CrossValidation:
-    """Defines class to execute cross validation.
+    """Defines class to execute time series cross validation.
 
     Cross validation is a useful technique to use multiple folds of the
     training and testing data to help optimize the performance of the
     model (e.g. hyperparameter tuning). For more info on cross validation, see
     https://en.wikipedia.org/wiki/Cross-validation_(statistics)
 
+    This procedure is also known in literature as a rolling origin evaluation.
+
     Attributes:
       train_percentage: A float for the percentage of data used for training.
       test_percentage: A float for the percentage of data used for testing.
@@ -949,8 +950,9 @@ class CrossValidation:
       data: :class:`kats.consts.TimeSeriesData` object to perform backtest on.
       params: Parameters to train model with.
       model_class: Defines the model type to use for backtesting.
-      rolling_window: A boolean flag to use the rolling window method instead
-        of the expanding window method (default False).
+      constant_train_size: A boolean flag to keep the train set size constant,
+          sliding it forward with each iteration instead of expanding the
+          train set with each iteration (default False).
       multi: A boolean flag to toggle multiprocessing (default True).
       results: List of tuples `(training_data, testing_data, trained_model,
         forecast_predictions)` storing forecast results.
@@ -970,16 +972,22 @@ class CrossValidation:
       >>> cv = CrossValidation(
             error_methods=all_errors,
             data=ts,
-            params=paramsparams,
+            params=params,
             train_percentage=50,
             test_percentage=25,
             num_folds=3,
             model_class=ARIMAModel,
             rolling_window=True
           )
-      >>> backtester.run_backtest()
+      >>> cv.run_cv()
       >>> mape = cv.get_error_value("mape") # Retrieve MAPE error
     """
+    size: int
+    results: List[Tuple[np.ndarray, np.ndarray, 'Model[Params]', np.ndarray]]
+    num_folds: int
+    errors: Dict[str, float]
+    raw_errors: List[np.ndarray]
+    _backtester: BackTesterParent
 
     def __init__(
         self,
@@ -989,10 +997,12 @@ class CrossValidation:
         train_percentage: float,
         test_percentage: float,
         num_folds: int,
+        # pyre-fixme[24]: Generic type `type` expects 1 type parameter, use
+        #  `typing.Type` to avoid runtime subscripting errors.
         model_class: Type,
-        rolling_window=False,
-        multi=True,
-    ):
+        constant_train_size: bool=False,
+        multi: bool=True,
+    ) -> None:
         logging.info("Initializing and validating parameter values")
         if train_percentage <= 0:
             logging.error("Non positive training percentage")
@@ -1023,7 +1033,7 @@ class CrossValidation:
         self.errors = {}
         self.raw_errors = []
 
-        if not rolling_window:
+        if not constant_train_size:
             self._backtester = BackTesterExpandingWindow(
                 error_methods,
                 data,
@@ -1078,7 +1088,7 @@ class CrossValidation:
             raise ValueError("Invalid error name")
 
 
-def _get_percent_size(size: int, percent: float) -> int:
+def _get_absolute_size(size: int, percent: float) -> int:
     """
     Returns absolute size corresponding to percentage of array of length size.
     """
