@@ -1,9 +1,10 @@
-#!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
+#
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+# pyre-unsafe
+
 """ Module that has parameter tuning classes for time series models.
 
 This module has a collection of classes. A subset of these classes are parameter tuning
@@ -17,6 +18,7 @@ such as a factory that creates search strategy objects.
 """
 
 import logging
+import math
 import time
 import uuid
 from abc import ABC, abstractmethod
@@ -35,16 +37,23 @@ from ax.core.outcome_constraint import OutcomeConstraint
 from ax.modelbridge.discrete import DiscreteModelBridge
 from ax.modelbridge.registry import Models
 from ax.runners.synthetic import SyntheticRunner
-from ax.service.utils.instantiation import (
-    outcome_constraint_from_str,
-    parameter_from_json,
-)
 from kats.consts import SearchMethodEnum
+from ax.service.utils.instantiation import InstantiationBase
 
 # Maximum number of worker processes used to evaluate trial arms in parallel
 MAX_NUM_PROCESSES = 50
 
 
+def compute_search_cardinality(params_space: List[Dict]) -> float:
+    """compute cardinality of search space params"""
+    # check if search space is infinite
+    is_infinite = any([param["type"] == "range" for param in params_space])
+    if is_infinite:
+        return math.inf
+    else:
+        return math.prod([len(param["values"]) for param in params_space])
+
+
 class Final(type):
     """A helper class to ensure a class cannot be inherited.
 
@@ -233,10 +242,10 @@ class TimeSeriesParameterTuning(ABC):
             "Parameter tuning search space dimensions: {}".format(parameters)
         )
         self.validate_parameters_format(parameters)
-        self.parameters = [parameter_from_json(parameter) for parameter in parameters]
+        self.parameters = [InstantiationBase.parameter_from_json(parameter) for parameter in parameters]
         self.outcome_constraints = (
             [
-                outcome_constraint_from_str(str_constraint)
+                InstantiationBase.outcome_constraint_from_str(str_constraint)
                 for str_constraint in outcome_constraints
             ]
             if outcome_constraints is not None
@@ -362,8 +371,6 @@ class TimeSeriesParameterTuning(ABC):
         # We run the most recent batch trial as we only run candidate trials
         self._exp.trials[max(self._exp.trials)].run()
         self._trial_data = Data.from_multiple_data(
-            # pyre-fixme[6]: Expected `Iterable[ax.core.data.Data]` for 1st param
-            #  but got `Iterable[ax.core.abstract_data.AbstractDataFrameData]`.
             [
                 self._trial_data,
                 self._exp.fetch_trials_data(trial_indices=[max(self._exp.trials)]),
@@ -450,8 +457,12 @@ class TimeSeriesParameterTuning(ABC):
         )
         if self.outcome_constraints:
             # Deduplicate entries for which there are outcome constraints
-            # pyre-ignore[16]: `None` has no attribute `index`.
-            armscore_df = armscore_df.loc[armscore_df.astype(str).drop_duplicates().index]
+            armscore_df = armscore_df.loc[
+                # pyre-ignore[16]: `None` has no attribute `index`.
+                armscore_df.astype(str)
+                .drop_duplicates()
+                .index
+            ]
             if legit_arms_only:
 
                 def filter_violating_arms(
@@ -479,9 +490,9 @@ class TimeSeriesParameterTuning(ABC):
 
                 filtered_arms = filter_violating_arms(
                     list(self._exp.arms_by_name.values()),
-                    # pyre-fixme[6]: Expected `Data` for 2nd param but got
-                    #  `AbstractDataFrameData`.
                     self._exp.fetch_data(),
+                    # pyre-fixme[6]: Expected `OptimizationConfig` for 3rd param but
+                    #  got `Optional[ax.core.optimization_config.OptimizationConfig]`.
                     self._exp.optimization_config,
                 )
                 armscore_df = armscore_df[
@@ -852,7 +863,10 @@ class BayesianOptSearch(TimeSeriesParameterTuning):
         )
         model_run = self._bayes_opt_model.gen(n=arm_count)
         self.generator_run_for_search_method(
-            evaluation_function=evaluation_function, generator_run=model_run
+            evaluation_function=evaluation_function,
+            # pyre-fixme[6]: Expected `DiscreteModelBridge` for 2nd param but got
+            #  `GeneratorRun`.
+            generator_run=model_run,
         )
 
 
